/*
 * This file is part of decoding
 * Copyright 2012-2013  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#define _POSIX_C_SOURCE 1

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <decoding/decoding.h>

#define RING_NAME fld
#define EXT_DEGREE 16
#include <decoding/rings/mpfq_gf2n_wrapper.c>
#include <decoding/algos.c>

void vec_print_list(FILE *out,fld_vec m,fld_vec *a,uma na) {
  uma i;
  for( i = 0 ; i < na ; i++ ) {
    fld_vec_print(out,a[i]);
    fprintf(stderr," at distance %d\n",fld_vec_hamming_distance(m,a[i]));
    /* fputc('\n',out); */
  }
}

void die(fld_vec m,fld_vec *a,uma na) {
  fprintf(stderr,"Error: cannot find message:\n");
  fld_vec_print(stderr,m); fputc('\n',stderr);
  if ( na > 0 ) {
    fprintf(stderr,"List returned:\n");
    vec_print_list(stderr,m,a,na);
  }
  exit(-1);
}

void sudan(uma n,uma k,uma percent,uma max) {
  uma i,j,rad,tau,nc;
  fld_vec m,y,e,*c;
  fld_rs_code rs;

  fld_vec_inits(m,y,e,NULL);
  fld_rs_code_init_with_support(rs,n,k);
  rad = sudan_max_errors(n,k);
  tau = (int)((double)percent * (double)rad / 100.0);
  printf("Sudan algorithm (x%d):\n"
         "      n = %d,\n"
         "      k = %d,\n"
         "    tau = %d, (%d%% of errors, max is %d).\n",
         max,n,k,tau,percent,rad);
  for( i = 0 ; i < max ; i++ ) {
    fld_rs_code_random_codeword(m,rs);
    fld_vec_random_hamming_weight(e,n,tau);
    fld_vec_add(y,m,e);
    nc = fld_rs_code_sudan_koetter(&c,rs,y,tau);
    if ( nc == 0 ) die(m,c,0);
    for( j = 0 ; j < nc ; j++ ) {
      if ( fld_vec_cmp(c[j],m) == 0 ) goto sudan_good;
    }
    die(m,c,nc);
sudan_good:;
    fld_vec_ptr_clear(c,nc);
  }
  fld_rs_code_clear(rs);
  fld_vec_clears(m,y,e,NULL);
  fputs("OK\n",stdout);
}

void guruswami_sudan(uma n,uma k,uma tau,uma max) {
  clock_t t0,t1;
  double tt;
  uma i,j,rad,nc,mult;
  
  fld_vec m,y,e,*c;
  fld_rs_code rs;

  fld_vec_inits(m,y,e,NULL);
  fld_rs_code_init_with_support(rs,n,k);
  fld_vec_print(stdout,rs[0].supp); fflush(stdout);
  rad = guruswami_sudan_max_errors(n,k);
  /*tau = (int)((double)percent * (double)rad / 100.0);*/
  mult = guruswami_sudan_mult(n,k,tau);
  
  printf("Guruswami-Sudan algorithm (x%d):\n"
         "      n = %d,\n"
         "      k = %d,\n"
         "    tau = %d, (max is %d).\n"
         "   mult = %d\n",
         max,n,k,tau,rad,mult);
  
  t0 = clock();
  for( i = 0 ; i < max ; i++ ) {
    fld_rs_code_random_codeword(m,rs);
    fld_vec_random_hamming_weight(e,n,tau);
    fld_vec_add(y,m,e);
    nc = fld_rs_code_guruswami_sudan_koetter(&c,rs,y,tau);
    if ( nc == 0 ) die(m,c,0);
    for( j = 0 ; j < nc ; j++ ) {
      if ( fld_vec_cmp(c[j],m) == 0 ) goto guruswami_sudan_good;
    }
    die(m,c,nc);
guruswami_sudan_good:;
    fld_vec_ptr_clear(c,nc);
  }
  t1 = clock();
  tt = (double)(t1 - t0) / (double)CLOCKS_PER_SEC;
  fld_rs_code_clear(rs);
  fld_vec_clears(m,y,e,NULL);
  printf("Time: %f\n",tt);
}

uma main(uma argc,char **argv) {
  uma n,k,tau,iter;
  if ( argc != 5 ) {
    fprintf(stderr,"usage: gf256_sugar_gs n k tau iter\n");
    return -1;
  }
  n = atoi(argv[1]);  
  k = atoi(argv[2]);  
  tau = atoi(argv[3]);
  iter = atoi(argv[4]);
  ll_random_seed(time(0));
  fld_ring_init();
  /* print_compile_flags(stdout); */
  /* sudan(66,16,100,100); */
  guruswami_sudan(n,k,tau,iter);
  return 0;
}

