/*
 * This file is part of decoding
 * Copyright 2012-2013  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#define _POSIX_C_SOURCE 1

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <decoding/decoding.h>

#define RING_NAME fld /* fld := GF(67) */
#define PRIME 67
#include <decoding/rings/gfp_word.c>
#include <decoding/algos.c>

void bench_dbpol_shift(uma dX,uma dY,uma n) {
  clock_t t0,t1;
  double tt;
  uma i;
  fld *f,*g,x,y;
  f = fld_ll_vec_malloc_random(dX * dY);
  g = fld_ll_mat_malloc_init(dX,dY);
  fld_init(x);
  fld_init(y);
  fld_random(x);
  fld_random(y);
  t0 = clock();
  for( i = 0 ; i < n ; i++ ) {
    fld_ll_dbpol_shift(g,f,dX,dY,x,y);
  }
  t1 = clock();
  fld_ll_mat_free(f,dX,dY);
  fld_ll_mat_free(g,dX,dY);
  fld_clear(y);
  fld_clear(x);
  tt = (double)(t1 - t0) / (double)CLOCKS_PER_SEC;
  printf("ll_dbpol_shift_naive: %f\n",tt);
}

void bench_dbpol_shift_naive(uma dX,uma dY,uma n) {
  clock_t t0,t1;
  double tt;
  uma i;
  fld *f,*g,x,y;
  f = fld_ll_mat_malloc_random(dX,dY);
  g = fld_ll_mat_malloc_init(dX,dY);
  fld_init(x);
  fld_init(y);
  fld_random(x);
  fld_random(y);
  t0 = clock();
  for( i = 0 ; i < n ; i++ ) {
    fld_ll_dbpol_shift_naive(g,f,dX,dY,x,y);
  }
  t1 = clock();
  fld_ll_mat_free(f,dX,dY);
  fld_ll_mat_free(g,dX,dY);
  fld_clear(y);
  fld_clear(x);
  tt = (double)(t1 - t0) / (double)CLOCKS_PER_SEC;
  printf("ll_dbpol_shift: %f\n",tt);
}

int main(int argc,char **argv) {
  uma dX,dY,n;
  if ( argc != 4 ) return -1;
  dX = atoi(argv[1]);
  dY = atoi(argv[2]);
  n = atoi(argv[3]);
  srand(time(0));
  fld_ring_init();
  bench_dbpol_shift_naive(dX,dY,n);
  /* bench_dbpol_shift(dX,dY,n); */
  return 0;
}

