/*
 * This file is part of decoding
 * Copyright 2012-2013  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#define _POSIX_C_SOURCE 1

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <decoding/decoding.h>

#define RING_NAME rng1
#define PRIME 257
#include <decoding/rings/gfp_float.c>
#include <decoding/algos.c>

#define RING_NAME rng2
#define PRIME 257
#include <decoding/rings/gfp_word.c>
#include <decoding/algos.c>

#define HELPER(R,OP,GHz) \
void __C(__C(R,_bench_vec_),OP)(FILE *out,uma e1,uma e2,uma iter) { \
  uma i,j,k,n; \
  clock_t start,end; \
  double total,single; \
 \
  R *a,*b,*c; \
  n = (1 << e2); \
  a = __C(R,_ll_vec_malloc_init)(n); \
  b = __C(R,_ll_vec_malloc_random)(n); \
  c = __C(R,_ll_vec_malloc_random)(n); \
  for( i = e1 ; i <= e2 ; i++ ) { \
    n = (1 << i); \
    start = clock(); \
    for( j = 0 ; j < iter ; j++ ) { \
      for( k = 0 ; k < n ; k++ ) { \
        __C(__C(R,_),OP)(a[k],b[k],c[k]); \
      } \
    } \
    end = clock(); \
    total = (double)(end - start); \
    total = total / (double)CLOCKS_PER_SEC; \
    total = total / (double)iter; \
    single = total / (double)n; \
    fprintf(out,"%ld    %e    %e    %f\n", \
                n,total,single,single * GHz); \
  } \
  n = (1 << e2); \
  __C(R,_ll_vec_free)(a,n); \
  __C(R,_ll_vec_free)(b,n); \
  __C(R,_ll_vec_free)(c,n); \
}

HELPER(rng1,add,3.2e9)
HELPER(rng2,add,3.2e9)
HELPER(rng1,mul,3.2e9)
HELPER(rng2,mul,3.2e9)

void print_op(FILE *out,const char *s) {
  fputs("\n          ",out);
  fputs(s,out);
  fputs("\n\n",out);
}

int main(void) {
  print_op(stdout,"ADDITION");
  fprintf(stdout,"float:\n"
                 "======\n\n");
  rng1_bench_vec_add(stdout,8,25,10);
  fprintf(stdout,"\n"
                 "unsigned int:\n"
                 "=============\n\n");
  rng2_bench_vec_add(stdout,8,25,10);

  print_op(stdout,"MULTIPLICATION");
  fprintf(stdout,"float:\n"
                 "======\n\n");
  rng1_bench_vec_mul(stdout,8,25,10);
  fprintf(stdout,"\n"
                 "unsigned int:\n"
                 "=============\n\n");
  rng2_bench_vec_mul(stdout,8,25,10);

  return 0;
}

