##
This file is part of decoding
Copyright (C) 2012-2013  Guillaume Quintin, INRIA

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
##

==OVERVIEW==

`decoding` provides some finite fields and finite rings. However
these implementations are not optimized. For efficiency it is
strongly recommanded to use external library like GMP or MPFQ.
To implement your own finite fields or rings or to wrap external
libraries see `decoding-rings`(3).

==DESCRIPTION==

The following finite fields and rings are provided by `decoding`.
If the wanted ring is implemented by `decoding/rings/wanted_ring.c`
you must do

``
#include <decoding/rings/wanted_ring.c>
#include <decoding/algos.c>
``

to use it. All the proposed rings are in the `include/decoding/rings`
directory. If you do not provide any name for your ring, one will
be set by default. It is recommended to use a name with the
`RING_NAME` macro.

``
#define RING_NAME myring
#include <decoding/rings/wanted_ring.c>
#include <decoding/algos.c>
``

Every ring in `decoding` *must* provide some information like
its characteristic or cardinality. Every variable is prefixed
by `R_` in what follows. To be used in your program the
`R_` prefix has to be replaced by the ring name, either
its default name or the name you gave with the `RING_NAME`
macro. The following *must* be provided by the ring.

> `R R_one;`
  The unity of the ring.

> `mpz_t R_char;`
  The characteristic of the ring.

> `uma R_uma_char;`
  The characteristic of the ring if it holds within a uma
  integer. Otherwise `R_uma_char` is `-1`.

> `mpz_t R_card;`
  The cardinality of the ring if it is finite.
  If the ring is not finite then `R_card` is zero.

> `uma R_ext_degree;`
  The degree of the ring.
  If the ring is an algebraic extension, that is
  `A[X] / (f(X))` for a ring `A` and a polynomial
  `f(X)` from `Z(A)[X]` of degree `d` then `R_ext_degree`
  will be equal to `d`.
  
  - If the ring is a finite field, `R_ext_degree` will be
    the degree of the finite field over the prime field.

More variables can be exported by the ring.
In that case they will be describe in what follows.

You can of course use several rings in the same program.
Suppose you want to use `field1.c` and `field2.c`.

``
#define RING_NAME F1
#include <decoding/rings/field1.c>
#include <decoding/algos.c>

#define RING_NAME F2
#include <decoding/rings/field2.c>
#include <decoding/algos.c>

int main(void) {
  F1 a,b;
  F2 x,y;

  F1_ring_init();
  F2_ring_init();
  F1_inits(a,b,NULL);
  F2_inits(x,y,NULL);

  /* do some stuff with F1 and F2 */
  
  F1_clears(a,b,NULL);
  F2_clears(x,y,NULL); 
  F1_ring_clear();
  F2_ring_clear();
  return 0;
}
``

In the example above two rings are used and their names
(`F1` and `F2`) are given with `RING_NAME`. Their names
are then used to prefixed all the functions involving
elements of both rings.

Note that before any use of the fields you *must* initialize them
with the `R_ring_init()` function. Depending on the ring you want to
use the latter function may take arguments. In this case, they are
documented in the appropriate sections below.

===`GF5.c`===

Implements GF(5). It is an example of implementation of a finite
field and contains the documentation on the making of finite rings
for `decoding`. See `decoding-rings`(3) for details.
It is not recommended to use this field, if you want to use
`GF(5)` please use the `gfp_word.c`.

