##
This file is part of decoding
Copyright (C) 2012-2013  Guillaume Quintin, INRIA

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
##

==OVERVIEW==

The functions presented below
provide a coherent calling interface. For example, they do not
require the user to manage the memory of the object manipulated
by the library.
The functions below shall be used by any user of the library and
shall be preferred over their low levels counterparts.

==DESCRIPTION==

To use the following functions with a given ring you must
*prefix* their names with the name you gave the ring you
are using. Also, the name of *all* types, vectors (`vec`)
and Reed-Solomon codes (`rs`) must be *prefixed* by the
name of the ring. Consider the following example which does the
addition of two vectors over `GF(101)`.

``
#include <decoding/decoding.h>

#define RING_NAME my_field
#define PRIME 101
#include <decoding/rings/gfp_word.c>

int main(void) {
  my_field_vec a,b,c;
  my_field_ring_init();
  my_field_inits(a,b,c,NULL);
  my_field_add(c,a,b);
  return 0;
}
``

We first declare three vectors `a`, `b` and `c` of type
`my_field_vec`.

``
my_field_vec a,b,c;
``

The type `my_field_vec` is in fact the
type `vec` corresponding to vectors, but prefixed with
`my_field_` which is the name we gave to `GF(101)`. Therefore
the type `my_field_vec` corresponds to vectors with coefficients
in `GF(101)`. The same reasoning holds for all the following
functions.

``
my_field_ring_init();
my_field_inits(a,b,c,NULL);
my_field_add(c,a,b); 
``

This mecanism allows one to use several rings at the same time.

``
#include <decoding/decoding.h>

#define RING_NAME gf101
#define PRIME 101
#include <decoding/rings/gfp_word.c>
#include <decoding/algos.c>

#define RING_NAME gf2_8
#define EXT_DEGREE 8
#include <decoding/rings/gf2n_word.c>
#include <decoding/algos.c>

int main(void) {
  gf101_vec a,b,c;
  gf2_8_vec u,v,w;
  
  gf101_ring_init();
  gf2_8_ring_init();

  gf101_inits(a,b,c,NULL);
  gf2_8_inits(u,v,w,NULL);
  gf101_add(c,a,b);
  gf2_8_add(w,u,v);
  
  return 0;
}
``
