##
This file is part of decoding
Copyright (C) 2013  Guillaume Quintin, INRIA

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
##

==OVERVIEW==

The low-level functions, prefixed by `ll`, do *not* provide a
coherent calling interface. They are directly accessible but
one must be careful, for example, to allocate the right amount
of memory for the result.

==DESCRIPTION==

The following holds for all low level functions except when
*explicitly* stated otherwise: the memory to store the result
must be allocated *before* the call to the low level
function ; the allocated memory must have *at least the size
to hold the final result* ; the memory for the result must
*not overlap* with the memory occupied by the input ; the
memory regions occupied by all or some elements of the input
can be *the same* ; the input is *not modified* by the function.

Note that many details must be taken care of when using
low level functions. For exemple if you have two polynomials
and add them with `ll_array_add` then it is up to you to
compute the degree of the resulting polynomial.

The prefix of functions indicates on what type of object it acts on.

- `ll_dbpol` for dense bivariate polynomials.

- `ll_upol` for univariate polynomials.

- `ll_vec` for vectors.

- `ll_mat` for matrices.

- `ll_array` for arrays.

- `ll_lili_vec` for linked lists of vectors.

===VECTORS AND ARRAYS===

Vectors and arrays are represented as simple C arrays of `R`,
concretely they are simple pointers `R*`. There is no practical
differences between vectors and arrays. A function prefixed
by `ll_vec` manipulates arrays of the same size, like
`ll_vec_add` or `ll_vec_sub` whereas a function prefixed by
`ll_array` can manipulates vectors of different sizes,
like `ll_array_add`.

===MATRICES===

Matrices are represented by a C array of `R`, namely they are
simple pointers `R*`. Thus you must give to functions
manipulating matrices the their sizes, the number of rows and
the numbers of columns.

===UNIVARIATE POLYNOMIALS===

Univariate polynomials are represented as simple C arrays
of `R`, concretely they are pointers `R*`. You must pass
to functions manipulating polynomials their sizes. The
*size* or *length* of a polynomial of degree `d` is
defined to be `d + 1`.

===DENSE BIVARIATE POLYNOMIALS===

Bivariate polynomials are represented as simple C arrays
of `R`, concretely they are pointers `R*`. You must pass
their size to functions manipulating them. The *size*
of a bivariate polynomial of degree `dx` in `X` and
`dy` in `Y` is defined to be the pair
(`dx + 1`,`dy + 1`).

===LINKED LIST OF VECTORS===

Linked lists of vectors are used for example by `ll_dbpol_Xroots`
to return the list of roots.

``
struct ll_lili_vec {
  R *a;
  int na;
  struct ll_lili_vec *next,*prev;
};
``

You can manipulate directly the `a` and `na` members where `na` is
thte size of `a`. The `prev` and `next` members shall not modified
directly, use the appropriate functions instead (see below).

===ALLOCATION FUNCTIONS===

All allocation function, usually containing `malloc` in their name,
return a pointer to allocated memory. In case the allocation fails,
`SIGABRT` is raised. All function that use an `*alloc`(3) function
from the standard library *shall* respect this behavior. You can
catch this signal with `sigaction`(2). By default it will terminate
silently your program.

==LIST OF LOW LEVEL FUNCTIONS==

