/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef _CEIL_LOG2_H
#define _CEIL_LOG2_H

/* This ugly (but still portable and ANSI) stuff is for
   statically computing ceil(log2(.)) so that gcc knows
   how to make good optimizations for loops.
   
   Here is the shell line that can generate this:

   for i in `seq 1 64`; do \
     j=`expr $i - 1`; \
     echo "#define __CLG$i(n) ( (n) > (1UL << $j) ? $i : __CLG$j(n) )"; \
   done

   I do not know of any 128-bits machine, so for now I stick with
   (max 64)-bits integers. */

#define __CLG1(n) ( (n) > (1UL << 0) ? 1 : 0 )
#define __CLG2(n) ( (n) > (1UL << 1) ? 2 : __CLG1(n) )
#define __CLG3(n) ( (n) > (1UL << 2) ? 3 : __CLG2(n) )
#define __CLG4(n) ( (n) > (1UL << 3) ? 4 : __CLG3(n) )
#define __CLG5(n) ( (n) > (1UL << 4) ? 5 : __CLG4(n) )
#define __CLG6(n) ( (n) > (1UL << 5) ? 6 : __CLG5(n) )
#define __CLG7(n) ( (n) > (1UL << 6) ? 7 : __CLG6(n) )
#define __CLG8(n) ( (n) > (1UL << 7) ? 8 : __CLG7(n) )
#define __CLG9(n) ( (n) > (1UL << 8) ? 9 : __CLG8(n) )
#define __CLG10(n) ( (n) > (1UL << 9) ? 10 : __CLG9(n) )
#define __CLG11(n) ( (n) > (1UL << 10) ? 11 : __CLG10(n) )
#define __CLG12(n) ( (n) > (1UL << 11) ? 12 : __CLG11(n) )
#define __CLG13(n) ( (n) > (1UL << 12) ? 13 : __CLG12(n) )
#define __CLG14(n) ( (n) > (1UL << 13) ? 14 : __CLG13(n) )
#define __CLG15(n) ( (n) > (1UL << 14) ? 15 : __CLG14(n) )
#define __CLG16(n) ( (n) > (1UL << 15) ? 16 : __CLG15(n) )
#define __CLG17(n) ( (n) > (1UL << 16) ? 17 : __CLG16(n) )
#define __CLG18(n) ( (n) > (1UL << 17) ? 18 : __CLG17(n) )
#define __CLG19(n) ( (n) > (1UL << 18) ? 19 : __CLG18(n) )
#define __CLG20(n) ( (n) > (1UL << 19) ? 20 : __CLG19(n) )
#define __CLG21(n) ( (n) > (1UL << 20) ? 21 : __CLG20(n) )
#define __CLG22(n) ( (n) > (1UL << 21) ? 22 : __CLG21(n) )
#define __CLG23(n) ( (n) > (1UL << 22) ? 23 : __CLG22(n) )
#define __CLG24(n) ( (n) > (1UL << 23) ? 24 : __CLG23(n) )
#define __CLG25(n) ( (n) > (1UL << 24) ? 25 : __CLG24(n) )
#define __CLG26(n) ( (n) > (1UL << 25) ? 26 : __CLG25(n) )
#define __CLG27(n) ( (n) > (1UL << 26) ? 27 : __CLG26(n) )
#define __CLG28(n) ( (n) > (1UL << 27) ? 28 : __CLG27(n) )
#define __CLG29(n) ( (n) > (1UL << 28) ? 29 : __CLG28(n) )
#define __CLG30(n) ( (n) > (1UL << 29) ? 30 : __CLG29(n) )
#define __CLG31(n) ( (n) > (1UL << 30) ? 31 : __CLG30(n) )
#define __CLG32(n) ( (n) > (1UL << 31) ? 32 : __CLG31(n) )
#define __CLG33(n) ( (n) > (1UL << 32) ? 33 : __CLG32(n) )
#define __CLG34(n) ( (n) > (1UL << 33) ? 34 : __CLG33(n) )
#define __CLG35(n) ( (n) > (1UL << 34) ? 35 : __CLG34(n) )
#define __CLG36(n) ( (n) > (1UL << 35) ? 36 : __CLG35(n) )
#define __CLG37(n) ( (n) > (1UL << 36) ? 37 : __CLG36(n) )
#define __CLG38(n) ( (n) > (1UL << 37) ? 38 : __CLG37(n) )
#define __CLG39(n) ( (n) > (1UL << 38) ? 39 : __CLG38(n) )
#define __CLG40(n) ( (n) > (1UL << 39) ? 40 : __CLG39(n) )
#define __CLG41(n) ( (n) > (1UL << 40) ? 41 : __CLG40(n) )
#define __CLG42(n) ( (n) > (1UL << 41) ? 42 : __CLG41(n) )
#define __CLG43(n) ( (n) > (1UL << 42) ? 43 : __CLG42(n) )
#define __CLG44(n) ( (n) > (1UL << 43) ? 44 : __CLG43(n) )
#define __CLG45(n) ( (n) > (1UL << 44) ? 45 : __CLG44(n) )
#define __CLG46(n) ( (n) > (1UL << 45) ? 46 : __CLG45(n) )
#define __CLG47(n) ( (n) > (1UL << 46) ? 47 : __CLG46(n) )
#define __CLG48(n) ( (n) > (1UL << 47) ? 48 : __CLG47(n) )
#define __CLG49(n) ( (n) > (1UL << 48) ? 49 : __CLG48(n) )
#define __CLG50(n) ( (n) > (1UL << 49) ? 50 : __CLG49(n) )
#define __CLG51(n) ( (n) > (1UL << 50) ? 51 : __CLG50(n) )
#define __CLG52(n) ( (n) > (1UL << 51) ? 52 : __CLG51(n) )
#define __CLG53(n) ( (n) > (1UL << 52) ? 53 : __CLG52(n) )
#define __CLG54(n) ( (n) > (1UL << 53) ? 54 : __CLG53(n) )
#define __CLG55(n) ( (n) > (1UL << 54) ? 55 : __CLG54(n) )
#define __CLG56(n) ( (n) > (1UL << 55) ? 56 : __CLG55(n) )
#define __CLG57(n) ( (n) > (1UL << 56) ? 57 : __CLG56(n) )
#define __CLG58(n) ( (n) > (1UL << 57) ? 58 : __CLG57(n) )
#define __CLG59(n) ( (n) > (1UL << 58) ? 59 : __CLG58(n) )
#define __CLG60(n) ( (n) > (1UL << 59) ? 60 : __CLG59(n) )
#define __CLG61(n) ( (n) > (1UL << 60) ? 61 : __CLG60(n) )
#define __CLG62(n) ( (n) > (1UL << 61) ? 62 : __CLG61(n) )
#define __CLG63(n) ( (n) > (1UL << 62) ? 63 : __CLG62(n) )
#define __CLG64(n) ( (n) > (1UL << 63) ? 64 : __CLG63(n) )

/* The actual macro to call */
#define CEIL_LOG2(n) __C(__CLG,__ULONG)(__CAST(unsigned long,n))

#endif /* _CEIL_LOG2_H */

