/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef _DECODING_H
#define _DECODING_H


/* When defined, __DECODING_A indicates that the static library
   is being compiled. I don't want to include unnecessary stuff
   like GMP and all the macros for the rings, generic, lowlevels
   and sugar functions so that the static library and, as a
   consequence, the programs in utils do not depend on unnecessary
   stuff. */

#ifdef __cplusplus
extern "C" {
#endif

/* I need all this */
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <math.h>
#include <limits.h>

#ifdef __cplusplus
}
#endif

/* I am NOT building the .a file */
#ifndef __DECODING_A
#include <gmp.h>
#endif

/* Also include the machine configuration
   generated by the Makefile */
#include <decoding/config.h>
#include <decoding/ceil_log2.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Optional dependencies */
#ifndef __DECODING_A
#ifdef __MULTI_THREADING
#include <omp.h>
#endif
#ifdef __GF2X
#include <gf2x.h>
#endif
#endif

/* common functions, from decoding.c */
extern int sudan_max_errors(int,int);
extern int sudan_list_size(int,int,int);
extern int guruswami_sudan_max_errors(int,int);
extern int guruswami_sudan_mult(int,int,int);
extern int guruswami_sudan_list_size(int,int,int,int);
extern int guruswami_sudan_errors(int,int,int);
extern int bitsize(unsigned long);
extern int half_log2(unsigned long);

#ifdef __cplusplus
}
#endif

/* some defines */
#define SUDAN_MAX_ERRORS             -1
#define GURUSWAMI_SUDAN_MAX_ERRORS   -1

/* useful macros */
#ifndef MAX
#define MAX(a,b) ((a) > (b) ? (a) : (b))
#endif
#ifndef MIN
#define MIN(a,b) ((a) < (b) ? (a) : (b))
#endif
#define XCHG(a,b,xchg) { (xchg) = (a); (a) = (b); (b) = (xchg); }
#ifdef __cplusplus
#define __CAST(a,b) static_cast<a>(b)
#else
#define __CAST(a,b) ((a)(b))
#endif
/* concatenation macro */
#define __C2(a,b) a##b
#define __C(a,b)  __C2(a,b)
/* stringification macro */
#define __S2(a) #a
#define __S(a) __S2(a)


/*************************************************

  From here, it is NOT for building the .a file

*************************************************/


#ifndef __DECODING_A

#ifdef __cplusplus
extern "C" {
#endif

/* Copied from mpfq: src/test/mpfq_test.c.meta */
extern char __gmp_rands_initialized;
extern gmp_randstate_t __gmp_rands;

#ifdef __cplusplus
}
#endif

/* Our sizeof function IN BITS */
#define __SZ(a) (sizeof(a) * __CHAR)

/* bench stuff */
#ifdef DECODING_BENCH
double __tt0,__tt1,__tt2,__tt3,__tt4,
       __tt5,__tt6,__tt7,__tt8,__tt9;
clock_t __cc0,__cc1;
#define __GO      { __cc0 = clock(); }
#define __STOP(a) { __cc1 = clock(); \
                    __tt ## a = (double)(__cc1 - __cc0) / \
                                (double)CLOCKS_PER_SEC; }
#define __ACC(a) { __cc1 = clock(); \
                    __tt ## a += (double)(__cc1 - __cc0) / \
                                 (double)CLOCKS_PER_SEC; }
#define __SHOW(a,msg) { printf(msg ": %fs\n",__tt ## a); }
#endif

#ifdef __cplusplus
namespace decoding {
#endif

/* generic functions */
#include <decoding/generic_decl.h>
#include <decoding/generic_code.c>

#ifdef __cplusplus
}
#endif

#endif /* __DECODING_A */

#endif /* _DECODING_H */

