/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `void ll_dbpol_eval_horner(R z,R *a,uma nax,uma nay,R x,R y);`
  Evaluate `a` of size (`nax`,`nay`) at (`x`,`y`).
  The Horner algorihtm is used.
*/

void ll_dbpol_eval_horner(R z,R *a,uma nax,uma nay,R x,R y) {
  uma i;
  R *tmp;
  tmp = ll_vec_malloc_init(nay);
  for( i = 0 ; i < nay ; i++ ) {
    ll_upol_eval_horner(tmp[i],&a[i * nax],nax,x);
  }
  ll_upol_eval_horner(z,tmp,nay,y);
  ll_vec_free(tmp,nay);
}

/**
> `void ll_dbpol_eval_horner_upol(R *r,uma *nr,R *a,uma nax,uma nay\
  ,R *b,uma nb);`
  Evaluate `a` of size (`nax`,`nay`) at `b` of size `nb`.
  The Horner algorithm is used.
*/

void ll_dbpol_eval_horner_upol(R *r,uma *nr,
                               R *a,uma nax,uma nay,R *b,uma nb)
{
  R *s,*t,*t0,*xchg;
  uma i,ns,nt0;
  s = r;
  nt0 = (nay - 1) * (nb - 1) + (nax - 1) + 1;
  t0 = ll_vec_malloc_init(nt0);
  t = t0;
  ll_vec_copy(s,&a[(nay - 1) * nax],nax);
  ns = nax;
  for( i = nay - 2 ; i >= 0 ; i-- ) {
    ll_upol_mul(t,s,ns,b,nb);
    ns += nb - 1;
    for( ; ns > 0 && R_iszero(t[ns - 1]) ; ns-- );
    ll_array_add(t,t,ns,&a[i * nax],nax);
    ns = MAX(ns,nax);
    xchg = s;
    s = t;
    t = xchg;
  }
  if ( s != r ) ll_vec_copy(r,s,ns);
  ll_vec_free(t0,nt0);
  nr[0] = ns;
}

