/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `void ll_dbpol_mul_naive(R *r,R *a,uma nax,uma nay,R *b,\
  uma nbx,uma nby);`
  Set `r` to `a * b` where `a` and `b` have size (`nax`,`nay`) and
  (`nbx`,`nby`).
  The schoolbook algorithm is used.
*/

void ll_dbpol_mul_naive(R *r,R *a,uma nax,uma nay,R *b,uma nbx,uma nby) {
  uma i,j,k,nry,nrx;
  R *tmp;

  nry = nay + nby - 1;
  nrx = nax + nbx - 1;
  tmp = ll_vec_malloc_init(nrx);
  for( i = 0 ; i < nry ; i++ ) {
    j = i - (nby - 1);
    if ( j < 0 ) j = 0;
    k = i - j;
    ll_vec_zero(&r[i * nrx],nrx);
    for( ; k >= 0 && j < nay ; j++,k-- ) {
      ll_upol_mul(tmp,&a[j * nax],nax,&b[k * nbx],nbx);
      ll_vec_add(&r[i * nrx],&r[i * nrx],tmp,nrx);
    }
  }
  ll_vec_free(tmp,nrx);
}

/**
> `void ll_dbpol_mul(R *r,R *a,uma nax,uma nay,R *b,\
  uma nbx,uma nby);`
  Set `r` to `a * b` where `a` and `b` have size (`nax`,`nay`) and
  (`nbx`,`nby`).
  By default `ll_dbpol_mul_naive` is called.
  You can provide a better algorithm with `MY_DBPOL_MUL`.
  See `decoding-defines`(3) for more details.
  Some rings can provide their own function.
  See `decoding-alphabets`(3) for more details.
*/

void ll_dbpol_mul(R *r,R *a,uma nax,uma nay,R *b,uma nbx,uma nby) {
#ifdef MY_DBPOL_MUL
  MY_DBPOL_MUL(r,a,nax,nay,b,nbx,nby);
#else
  ll_dbpol_mul_naive(r,a,nax,nay,b,nbx,nby);
#endif
}

