/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `void ll_dbpol_shift_naive(R *r,R *a,uma nax,uma nay,R x,R y);`
  Set `r` to `a(X + x,Y + y)` where `a` has size (`nax`,`nay`).
  The horner algorithm is used.
*/

void ll_dbpol_shift_naive(R *r,R *a,uma nax,uma nay,R x,R y) {
  uma i;
  R *buf,*tmp,*atmp;
  buf = ll_vec_malloc_init(2 * nax * nay);
  atmp = buf;
  tmp = &buf[nax * nay];
  for( i = 0 ; i < nay ; i++ ) {
    ll_upol_shift_naive(&atmp[i * nax],&a[i * nax],nax,x);
  }
  ll_mat_transpose(tmp,atmp,nay,nax);
  for( i = 0 ; i < nax ; i++ ) {
    ll_upol_shift_naive(&atmp[i * nay],&tmp[i * nay],nay,y);
  }
  ll_mat_transpose(r,atmp,nax,nay);
  ll_vec_free(buf,2 * nax * nay);
}

/**
> `void ll_dbpol_shift(R *r,R *a,uma nax,uma nay,R x,R y);`
  Set `r` to `a(X + x,Y + y)` where `a` has size (`nax`,`nay`).
  You can provide your own function with `MY_DBPOL_SHIFT`.
  If your ring has caracteristic 2 (`RING_COMM_CHAR2`) then
  `ll_dbpol_shift_car2` is called and `ll_dbpol_shift_fast`
  otherwise.
*/

void ll_dbpol_shift(R *r,R *a,uma nax,uma nay,R x,R y) {
#ifdef MY_DBPOL_SHIFT
  MY_DBPOL_SHIFT(r,a,nax,nay,x,y);
#else
#ifdef RING_COMM_CHAR2
  ll_dbpol_shift_car2(r,a,nax,nay,x,y);
#else
  ll_dbpol_shift_fast(r,a,nax,nay,x,y);
#endif
#endif
}

