/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `void ll_mat_copy(R *r,R *a,uma ra,uma ca);`
  Do a deep copy of `a` of size (`ar`,`ac`) to `r`.
*/

void ll_mat_copy(R *r,R *a,uma ra,uma ca) {
  ll_vec_copy(r,a,ra * ca);
}

/**
> `R *ll_mat_malloc_init(uma rm,uma cm);`
  Return a pointer to a region of (`rm x cm`) `R`'s.
*/

R *ll_mat_malloc_init(uma rm,uma cm) {
  return ll_vec_malloc_init(rm * cm);
}

/**
> `R *ll_mat_malloc_random(uma mr,uma mc);`
  Return a pointer to a region of (`rm x cm`) random `R`'s.
*/

R *ll_mat_malloc_random(uma rm,uma cm) {
  return ll_vec_malloc_random(rm * cm);
}

/**
> `void ll_mat_free(R *m,uma rm,uma cm);`
  Free `m` of size `rm x cm`.
*/

void ll_mat_free(R *m,uma rm,uma cm) {
  ll_vec_free(m,rm * cm);
}

/**
> `void ll_mat_transpose(R *r,R *a,uma ra,uma ca);`
  Set `r` to be the transpose of `a` which has size (`ra`,`ca`).
*/

void ll_mat_transpose(R *r,R *a,uma ra,uma ca) {
  uma i,j;
  for( i = 0 ; i < ra ; i++ ) {
    for( j = 0 ; j < ca ; j++ ) {
      R_copy(r[j * ra + i],a[i * ca + j]);
    }
  }
}

/**
> `void ll_mat_print(FILE *out,R *a,uma ra,uma ca);`
  Pruma to `out` the matrix `a` of size (`ra`,`ca`).
*/

void ll_mat_print(FILE *out,R *a,uma ra,uma ca) {
  uma i;
  for( i = 0 ; i < ra ; i++ ) {
    ll_vec_print(out,&a[i * ca],ca);
    if ( i < (ra - 1) ) fputc('\n',out);
  }
}

/**
> `void ll_mat_put(R *r,uma cr,uma i0,uma j0,\
R *a,uma ra,uma ca);`
  Put `a` of size (`ra`,`ca`) into `r` with `cr` columns at
  position (`i0`,`j0`).
*/

void ll_mat_put(R *r,uma cr,uma i0,uma j0,
                R *a,uma ra,uma ca)
{
  uma i,j;
  for( i = 0 ; i < ra ; i++ ) {
    for( j = 0 ; j < ca ; j++ ) {
      R_copy(r[(i + i0) * cr + (j + j0)],a[i * ca + j]);
    }
  }
}

