/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

void ll_dbpol_mul_kronecker_word_encode(mp_limb_t *r,R *a,
                                        uma nax,uma nay,
                                        uma size,uma bits)
{
#ifdef RING_MODN_WORD
  uma i,j;
  for( j = 0 ; j < nay ; j++ ) {
    for( i = 0 ; i < nax ; i++ ) {
      uma off,sh,d;
      d = j * size + i;
      off = d * bits / GMP_NUMB_BITS;
      sh = (d * bits) % GMP_NUMB_BITS;
      ll_put_word_into_mp_limb(&r[off],a[j * nax + i],sh);
    }
  }
#else
  (void)r;
  (void)a;
  (void)nax;
  (void)nay;
  (void)size;
  (void)bits;
#endif
}

void ll_dbpol_mul_kronecker_word_decode(R *r,uma nrx,uma nry,
                                        mp_limb_t *sp,uma bits)
{
#ifdef RING_MODN_WORD
  uma i,j;
  if ( bits <= GMP_NUMB_BITS ) {
    mp_limb_t mask;
    ll_get_mask_1(&mask,bits);
    for( j = 0 ; j < nry ; j++ ) {
      for( i = 0 ; i < nrx ; i++ ) {
        uma off,sh,d;
        d = j * nrx + i;
        off = d * bits / GMP_NUMB_BITS;
        sh = (d * bits) % GMP_NUMB_BITS;
        ll_get_word_from_mp_limb_1(r[d],&sp[off],sh,mask);
      }
    }
  }
  else {
    uma n;
    mp_limb_t *mask;
    n = bits / GMP_NUMB_BITS;
    if ( (bits % GMP_NUMB_BITS) > 0 ) n++;
    n++; /* n limbs can hold within at most (n + 1) limbs */
    mask = __CAST(mp_limb_t*,ll_malloc(n * sizeof(mp_limb_t)));
    ll_get_mask_n(mask,bits);
    for( j = 0 ; j < nry ; j++ ) {
      for( i = 0 ; i < nrx ; i++ ) {
        uma off,sh,d;
        d = j * nrx + i;
        off = d * bits / GMP_NUMB_BITS;
        sh = (d * bits) % GMP_NUMB_BITS;
        ll_get_word_from_mp_limb_n(r[d],&sp[off],n,sh,mask);
      }
    }
    ll_free(mask);
  }
#else
  (void)r;
  (void)nrx;
  (void)nry;
  (void)sp;
  (void)bits;
#endif
}

void ll_dbpol_mul_kronecker_word(R *r,R *a,uma nax,uma nay,
                                      R *b,uma nbx,uma nby)
{
#ifdef RING_MODN_WORD
  uma bits,ns1p,ns2p,nrx,nry,size;
  mp_limb_t *mem,*s1p,*s2p,*rp;
  
  size = nax + nbx - 1;
  bits = 2 * bitsize((unsigned long)R_characteristic) + 2 +
             bitsize((unsigned long)MIN(nay,nby) *
                     (unsigned long)size) + 2;
  ns1p = size * nay * bits / GMP_NUMB_BITS + 2;
  ns2p = size * nby * bits / GMP_NUMB_BITS + 2;
  mem = __CAST(mp_limb_t*,
               ll_calloc(2 * (ns1p + ns2p) * sizeof(mp_limb_t)));
  s1p = mem;
  s2p = &mem[ns1p];
  rp = &mem[ns1p + ns2p];
  ll_dbpol_mul_kronecker_word_encode(s1p,a,nax,nay,size,bits);
  ll_dbpol_mul_kronecker_word_encode(s2p,b,nbx,nby,size,bits);
  if ( ns1p > ns2p ) {
    mpn_mul(rp,s1p,ns1p,s2p,ns2p);
  }
  else {
    mpn_mul(rp,s2p,ns2p,s1p,ns1p);
  }
  nrx = nax + nbx - 1;
  nry = nay + nby - 1;
  ll_dbpol_mul_kronecker_word_decode(r,nrx,nry,rp,bits);
  ll_free(mem);
#else
  ll_dbpol_mul_naive(r,a,nax,nay,b,nbx,nby);
#endif
}

