/*
 * This file is part of decoding
 * Copyright 2013  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `void ll_upol_pthroot_naive(R *r,uma *nr,R *a,uma na);`
  Set `r` of size `nr` to be the `p`-th root of `a` of size `na`
  where `p` is the characteristic of the ring. If the ring has
  characteristic zero then `r = a`.
  Note that this function assumes that `a` is actually a `p`-th
  power. If it is not the case thte result is undefined.
  A naive algorithm is used.
*/

void ll_upol_pthroot_naive(R *r,uma *nr,R *a,uma na) {
  uma i,p,n;
#ifdef CHARACTERISTIC_UMA
  p = __CAST(uma,R_characteristic);
#else
#ifdef CHARACTERISTIC_MPZ
  if ( mpz_cmp_ui(R_characteristic,
                  __CAST(unsigned long,uma_max)) < 0 )
  {
    p = __CAST(uma,mpz_get_si(R_characteristic));
  }
#else
  ll_die("missing CHARACTERISTIC_UMA or CHARACTERISTIC_MPZ");
#endif
#endif
  if ( p == 0 ) {
    ll_vec_copy(r,a,na);
    nr[0] = na;
    return;
  }
  n = (na - 1) / p + 1;
  for( i = 0 ; i < n ; i++ ) {
    R_pthroot(r[i],a[i * p]);
  }
  nr[0] = n;
}

/**
> `void ll_upol_pthroot(R *r,uma *nr,R *a,uma na);`
  Set `r` of size `nr` to be the `p`-th root of `a` of size `na`
  where `p` is the characteristic of the ring. If the ring has
  characteristic zero then `r = a`.
  Note that this function assumes that `a` is actually a `p`-th
  power. If it is not the case thte result is undefined.
*/

void ll_upol_pthroot(R *r,uma *nr,R *a,uma na) {
#ifdef MY_UPOL_PTHROOT
  MY_UPOL_PTHROOT(r,nr,a,na);
#else
  ll_upol_pthroot_naive(r,nr,a,na);
#endif
}

