/*
 * This file is part of decoding
 * Copyright 2013  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/* von zur Gathen, Second Edition,
   Algorithm 4.8 page 73 */

/**
> `void ll_upol_binpow_mod(R *r,uma *nr,R *a,uma na,mpz_t e,R *m,\
uma nm);`
  Set `r` to `a(X) ^ e mod m` where `a` has size `na` and `m` has
  size `nm` and set `nr` to the size of `r`. Note that `e` must
  be a GMP integer.
  The repeated squaring algorithm is used.
*/

void ll_upol_binpow_mod(R *r,uma *nr,R *a,uma na,mpz_t e,R *m,uma nm) {
  R *a0,*b,*tmp;
  uma na0,nb,nb2,k,ntmp,ntmp2;
 
  if ( !mpz_cmp_si(e,0) ) {
    R_castint(r[0],1);
    nr[0] = 1;
    return;
  }
  ntmp2 = crit_mul(2,nm);
  nb2 = MAX(ntmp2,na);
  b = ll_vec_malloc_init(nb2);
  ll_upol_rem(b,&nb,a,na,m,nm);
  if ( ll_upol_iszero(b,nb) ) {
    R_zero(r[0]);
    nr[0] = 1;
    goto ll_upol_binpow_mod_free_b;
  }
  tmp = ll_vec_malloc_init(ntmp2);
  na0 = nb;
  a0 = ll_vec_malloc_copy(b,nb);
  for( k = mpz_sizeinbase(e,2) - 2 ; k >= 0 ; k-- ) {
    ll_upol_square(tmp,b,nb);
    ntmp = 2 * nb - 1;
    ll_upol_rem(b,&nb,tmp,ntmp,m,nm);
    if ( mpz_tstbit(e,k) ) {
      ll_upol_mul(tmp,b,nb,a0,na0);
      ntmp = nb + na0 - 1;
      ll_upol_rem(b,&nb,tmp,ntmp,m,nm);
    }
  }
  nr[0] = nb;
  ll_vec_copy(r,b,nb);
  ll_vec_free(a0,na0);
  ll_vec_free(tmp,ntmp2);
  
ll_upol_binpow_mod_free_b:
  ll_vec_free(b,nb2);
}

