/*
 * This file is part of decoding
 * Copyright 2013  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/* equal-degree factorization, Algorithm 14.10 page 377,
   Gathen, Modern Computer Algebra, 2nd edition */

/**
> `struct ll_vec_lili *ll_upol_eqdegfact(R *a,uma na,uma d);`
  Return all the irredusible factors of `a` of size `na`
  of degree `d`.
  Note that all irreducible factors of `a` *must* have degree `d`
  and that `a` *must* have degree `na - 1`.
*/

struct ll_vec_lili *ll_upol_eqdegfact(R *a,uma na,uma d) {
#ifdef MY_UPOL_EQDEGFACT
  return MY_UPOL_EQDEGFACT(a,na,d);
#else
#ifdef RING_FF
  R *f,*g;
  uma nf,nf2,ng;
  struct ll_vec_lili *ret,*last,*rest;

  if ( na == d + 1 ) {
    ret = ll_vec_lili_node_malloc_copy(a,na);
    return ret;
  }
  nf2 = na;
  f = ll_vec_malloc_init(nf2);
  ll_upol_eqdegsplit(f,&nf,a,na,d);
  if ( nf == d + 1 ) {
    ret = ll_vec_lili_node_malloc_copy(f,nf);
    last = ret;
  }
  else {
    ret = ll_upol_eqdegfact(f,nf,d);
    for( last = ret ; last->next ; last = last->next );
  }
  ng = na - nf + 1;
  g = ll_vec_malloc_init(ng);
  ll_upol_quo(g,a,na,f,nf);
  if ( ng == d + 1 ) {
    rest = ll_vec_lili_node_malloc_copy(g,ng);
  }
  else {
    rest = ll_upol_eqdegfact(g,ng,d);
  }
  last->next = rest;
  rest->prev = last;
  ll_vec_free(f,nf2);
  ll_vec_free(g,ng);
  return ret;
#else
  (void)a;
  (void)na;
  (void)d;
  return NULL;
#endif
#endif
}

