/*
 * This file is part of decoding
 * Copyright 2012-2013  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `void ll_upol_gcd_naive(R *r,uma *nr,R *a,uma na,R *b,uma nb);`
  Set `r` of size `nr` to be GCD of `a` of size `na` and `b` of
  size `nb`.
  Note that the degree of `a` *must* be `na - 1` and the degree of
  `b` must be `nb - 1`.
  The schoolbook algorithm is used.
*/

void ll_upol_gcd_naive(R *r,uma *nr,R *a,uma na,R *b,uma nb) {
  R *f0,*f,*g0,*g,*tmp;
  uma nf0,nf,ng0,ng,ntmp;

  if ( na >= nb ) {
    f = ll_vec_malloc_copy(a,na); nf = na;
    g = ll_vec_malloc_copy(b,nb); ng = nb;
  }
  else {
    g = ll_vec_malloc_copy(a,na); ng = na;
    f = ll_vec_malloc_copy(b,nb); nf = nb;
  }
  f0 = f; nf0 = nf;
  g0 = g; ng0 = ng;
  for( ; !ll_upol_iszero(g,ng) ; ) {
    ll_upol_rem(f,&nf,f,nf,g,ng);
    XCHG(f,g,tmp);
    XCHG(nf,ng,ntmp);
    /*
    tmp = f; f = g; g = tmp;
    ntmp = nf; nf = ng; ng = ntmp;
    */
  }
  nr[0] = nf;
  ll_vec_copy(r,f,nf);
  ll_vec_free(f0,nf0);
  ll_vec_free(g0,ng0);
}

/**
> `void ll_upol_gcd(R *r,uma *nr,R *a,uma na,R *b,uma nb);`
  Set `r` of size `nr` to be GCD of `a` of size `na` and `b` of
  size `nb`.
  Note that the degree of `a` *must* be `na - 1` and the degree of
  `b` must be `nb - 1`.
*/

void ll_upol_gcd(R *r,uma *nr,R *a,uma na,R *b,uma nb) {
#ifdef MY_UPOL_GCD
  MY_UPOL_GCD(r,nr,a,na,b,nb);
#else
  ll_upol_gcd_naive(r,nr,a,na,b,nb);
#endif
}

