/*
 * This file is part of decoding
 * Copyright 2012-2013  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `void ll_upol_interpolation_naive(R *r,uma *nr,R *x,R *y,uma n);`
  Set `r` of size `nr` to be the polynomial of degree at most
  `n - 1` which interpolates the `n` points `(x[i],y[i])`,
  in other words such that `r(x[i]) = y[i]`.
  The naive algorithm with Lagrange polynomials is used.
*/

void ll_upol_interpolation_naive(R *r,uma *nr,R *x,R *y,uma n) {
  R coeff,xi,*tmp,*L,*prod;
  uma ntmp0,nL0,i,j,nprod;

  ntmp0 = n;
  tmp = ll_vec_malloc_init(ntmp0);
  nL0 = n + 1;
  L = ll_vec_malloc_init(nL0);
  nprod = nL0;
  prod = ll_vec_malloc_init(nprod);
  ll_upol_prod_affine_neg(prod,x,n);
  R_inits(coeff,xi,NULL);
  ll_vec_zero(r,n);
  for( i = 0 ; i < n ; i++ ) {
    R_copy(xi,x[i]);
    for( j = 0 ; j < i ; j++ ) {
      R_sub(tmp[j],xi,x[j]);
    }
    for( j++ ; j < n ; j++ ) {
      R_sub(tmp[j - 1],xi,x[j]);
    }
    ll_vec_prod(coeff,tmp,n - 1);
    R_div(coeff,y[i],coeff);
    R_neg(xi,xi);
    ll_upol_div_by_affine(L,prod,nprod,xi);
    ll_vec_add_mul_by_cte(r,L,n,coeff);
  }
  R_clears(coeff,xi,NULL);
  ll_vec_free(tmp,ntmp0);
  ll_vec_free(L,nL0);
  ll_vec_free(prod,nprod);
  nr[0] = ll_vec_true_size(r,n);
}

/**
> `void ll_upol_interpolation(R *r,uma *nr,R *x,R *y,uma n);`
  Set `r` of size `nr` to be the polynomial of degree at most
  `n - 1` which interpolates the `n` points `(x[i],y[i])`,
  in other words such that `r(x[i]) = y[i]`.
*/

void ll_upol_interpolation(R *r,uma *nr,R *x,R *y,uma n) {
#ifdef MY_UPOL_INTERPOLATION
  MY_UPOL_INTERPOLATION(r,nr,x,y,n);
#else
  ll_upol_interpolation_naive(r,nr,x,y,n);
#endif
}

