/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `void ll_upol_mul_naive(R *r,R *a,uma na,R *b,uma nb);`
  Set `r` to `a * b` where `a` has size `na` and `b` has
  size `nb`.
  The shoolbook algorithm is used.
*/

void ll_upol_mul_naive(R *r,R *a,uma na,R *b,uma nb) {
  uma i,j,k,nr;

  nr = na + nb - 1;
  for( i = 0 ; i < nr ; i++ ) {
    j = i - (nb - 1);
    if ( j < 0 ) j = 0;
    k = i - j;
    R_zero(r[i]);
    for( ; k >= 0 && j < na ; j++,k-- ) {
      R_add_mul(r[i],a[j],b[k]);
    }
  }
}

/**
> `void ll_upol_mul(R *r,R *a,uma na,R *b,uma nb);`
  Set `r` to `a * b` where `a` has size `na` and `b` has
  size `nb`.
  By default the shcoolbook algorithm is used,
  `ll_upol_naive` is called.
  You can provide your
  own multiplication function with `MY_UPOL_MUL`, see
  `decoding-defines`(3) for more details.
*/

void ll_upol_mul(R *r,R *a,uma na,R *b,uma nb) {
#ifdef MY_UPOL_MUL
  MY_UPOL_MUL(r,a,na,b,nb);
#elif defined(RING_MODN_WORD)
  ll_upol_mul_kronecker_modn_word(r,a,na,b,nb);
#elif defined(RING_CHAR2_WORD) && defined(__GF2X)
  ll_upol_mul_kronecker_char2_word(r,a,na,b,nb);
#else
  ll_upol_mul_naive(r,a,na,b,nb);
#endif
}

