/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `void ll_upol_mul_by_affine(R *r,R *a,uma na,R b)`
  Set `r` to be `a * (X + b)` where `a` is a univariate
  polynomial of size `na`.
  You can have `r == a`.
*/

void ll_upol_mul_by_affine(R *r,R *a,uma na,R b) {
  uma i;
  if ( r == a ) {
    R tmp0,tmp1;
    R_init(tmp0);
    R_init(tmp1);
    R_copy(tmp0,r[0]);
    R_mul(r[0],r[0],b);
    for( i = 1 ; i < na ; i++ ) {
      R_copy(tmp1,r[i]);
      R_mul(r[i],r[i],b);
      R_add(r[i],r[i],tmp0);
      R_copy(tmp0,tmp1);
    }
    R_copy(r[na],tmp0);
    R_clear(tmp0);
    R_clear(tmp1);
  }
  else {
    R_mul(r[0],a[0],b);
    for( i = 1 ; i < na ; i++ ) {
      R_copy(r[i],a[i - 1]);
      R_add_mul(r[i],a[i],b);
    }
    R_copy(r[na],a[na - 1]);
  }
}

