/*
 * This file is part of decoding
 * Copyright 2013  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

void ll_put_word_into_unsigned_long(unsigned long *r,R a,uma sh) {
#if defined(RING_CHAR2_WORD) && defined(__GF2X)
  unsigned long tmp;
  tmp = __CAST(unsigned long,a[0]);
  r[0] |= tmp << sh;
  if ( sh > 0 ) {
    r[1] |= tmp >> (__ULONG - sh);
  }
#else
  (void)r;
  (void)a;
  (void)sh;
#endif
}

void ll_get_word_from_unsigned_long_1(R r,unsigned long *a,uma sh,
                                      unsigned long mask)
{
#if defined(RING_CHAR2_WORD) && defined(__GF2X)
  unsigned long tmp;
  tmp = a[0] >> sh;
  if ( sh > 0 ) {
    tmp |= a[1] << (__ULONG - sh);
  }
  tmp &= mask;
  r[0] = __CAST(R_content,
           char2_word_reduce(tmp,
             __CAST(unsigned long,R_irr_poly),
             __CAST(unsigned long,R_ext_degree)));
#else
  (void)r;
  (void)a;
  (void)sh;
  (void)mask;
#endif
}

void ll_upol_mul_kronecker_char2_word_encode(unsigned long *r,
                                             R *a,uma na,
                                             uma bits)
{
#if defined(RING_CHAR2_WORD) && defined(__GF2X)
  uma i;
  for( i = 0 ; i < na ; i++ ) {
    uma off,sh;
    off = i * bits / __ULONG;
    sh = (i * bits) % __ULONG;
    ll_put_word_into_unsigned_long(&r[off],a[i],sh);
  }
#else
  (void)r;
  (void)a;
  (void)na;
  (void)bits;
#endif
}

void ll_upol_mul_kronecker_char2_word_decode(R *r,uma nr,
                                             unsigned long *sp,
                                             uma bits)
{
#if defined(RING_CHAR2_WORD) && defined(__GF2X)
  uma i;
  unsigned long mask;
  ll_get_mask_1(&mask,bits);
  for( i = 0 ; i < nr ; i++ ) {
    uma off,sh;
    off = i * bits / __ULONG;
    sh = (i * bits) % __ULONG;
    ll_get_word_from_unsigned_long_1(r[i],&sp[off],sh,mask);
  }
#else
  (void)r;
  (void)nr;
  (void)sp;
  (void)bits;
#endif
}

void ll_upol_mul_kronecker_char2_word(R *r,R *a,uma na,R *b,uma nb) {
#if defined(RING_CHAR2_WORD) && defined(__GF2X)
  uma bits,ns1p,ns2p,nr;
  unsigned long *mem,*s1p,*s2p,*rp;

  bits = 2 * R_ext_degree - 1;
  ns1p = na * bits / __ULONG + 1;
  ns2p = nb * bits / __ULONG + 1;
  mem = __CAST(unsigned long*,
               ll_calloc((1 + 2 * (ns1p + ns2p))
                         * sizeof(unsigned long)));
  s1p = mem;
  s2p = &mem[ns1p];
  rp = &mem[ns1p + ns2p];
  ll_upol_mul_kronecker_char2_word_encode(s1p,a,na,bits);
  ll_upol_mul_kronecker_char2_word_encode(s2p,b,nb,bits);
  gf2x_mul(rp,s1p,ns1p,s2p,ns2p);
  nr = na + nb - 1;
  ll_upol_mul_kronecker_char2_word_decode(r,nr,rp,bits);
  ll_free(mem);
#else
  ll_upol_mul_naive(r,a,na,b,nb);
#endif
}

