/*
 * This file is part of decoding
 * Copyright 2012-2013  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

void ll_put_word_into_mp_limb(mp_limb_t *r,R a,uma sh) {
#ifdef RING_MODN_WORD
  mp_limb_t tmp;
  tmp = (mp_limb_t)(a[0]);
  r[0] |= (tmp << sh) & GMP_NUMB_MASK;
  if ( sh > 0 ) {
    r[1] |= (tmp >> (sizeof(mp_limb_t) * 8 - sh)) & GMP_NUMB_MASK;
  }
#else
  (void)r;
  (void)a;
  (void)sh;
#endif
}

void ll_get_word_from_mp_limb_1(R r,mp_limb_t *a,uma sh,
                                mp_limb_t mask)
{
#ifdef RING_MODN_WORD
  mp_limb_t tmp;
  tmp = a[0] >> sh;
  if ( sh > 0 ) {
    tmp |= a[1] << (GMP_LIMB_BITS - sh);
  }
  tmp &= mask;
  r[0] = __CAST(R_content,tmp % (mp_limb_t)R_uma_char);
#else
  (void)r;
  (void)a;
  (void)sh;
  (void)mask;
#endif
}

void ll_get_word_from_mp_limb_n(R r,mp_limb_t *a,uma n,uma sh,
                                mp_limb_t *mask)
{
#ifdef RING_MODN_WORD
#define HELPER ((sizeof(uma) * 8 + (2 * GMP_NUMB_BITS) + 4) \
                / GMP_NUMB_BITS + 1)
  uma i;
  mp_limb_t rem,tmp0[HELPER],tmp1[HELPER];
  for( i = 0 ; i < n ; i++ ) {
    tmp0[i] = a[i];
  }
  if ( sh == 0) {
    ll_mpn_and_n(tmp0,mask,n);
    rem = mpn_mod_1(tmp0,n,(mp_limb_t)R_uma_char);
    r[0] = (R_content)rem;
  }
  else {
    mpn_rshift(tmp1,tmp0,n,sh);
    ll_mpn_and_n(tmp1,mask,n);
    rem = mpn_mod_1(tmp1,n,(mp_limb_t)R_uma_char);
    r[0] = __CAST(R_content,rem);
  }
#undef HELPER
#else
  (void)r;
  (void)a;
  (void)n;
  (void)sh;
  (void)mask;
#endif
}

void ll_upol_mul_kronecker_modn_word_encode(mp_limb_t *r,R *a,uma na,
                                            uma bits)
{
#ifdef RING_MODN_WORD
  uma i;
  for( i = 0 ; i < na ; i++ ) {
    uma off,sh;
    off = i * bits / GMP_NUMB_BITS;
    sh = (i * bits) % GMP_NUMB_BITS;
    ll_put_word_into_mp_limb(&r[off],a[i],sh);
  }
#else
  (void)r;
  (void)a;
  (void)na;
  (void)bits;
#endif
}

void ll_upol_mul_kronecker_modn_word_decode(R *r,uma nr,mp_limb_t *sp,
                                            uma bits)
{
#ifdef RING_MODN_WORD
  uma i;
  if ( bits <= GMP_NUMB_BITS ) {
    mp_limb_t mask;
    ll_get_mask_1(&mask,bits);
    for( i = 0 ; i < nr ; i++ ) {
      uma off,sh;
      off = i * bits / GMP_NUMB_BITS;
      sh = (i * bits) % GMP_NUMB_BITS;
      ll_get_word_from_mp_limb_1(r[i],&sp[off],sh,mask);
    }
  }
  else {
    uma n;
    mp_limb_t *mask;
    n = bits / GMP_NUMB_BITS;
    if ( (bits % GMP_NUMB_BITS) > 0 ) n++;
    n++; /* n limbs can hold within at most (n + 1) limbs */
    mask = __CAST(mp_limb_t*,ll_malloc(n * sizeof(mp_limb_t)));
    ll_get_mask_n(mask,bits);
    for( i = 0 ; i < nr ; i++ ) {
      uma off,sh;
      off = i * bits / GMP_NUMB_BITS;
      sh = (i * bits) % GMP_NUMB_BITS;
      ll_get_word_from_mp_limb_n(r[i],&sp[off],n,sh,mask);
    }
    ll_free(mask);
  }
#else
  (void)r;
  (void)nr;
  (void)sp;
  (void)bits;
#endif
}

void ll_upol_mul_kronecker_modn_word(R *r,R *a,uma na,R *b,uma nb) {
#ifdef RING_MODN_WORD
  uma bits,ns1p,ns2p,nr;
  mp_limb_t *mem,*s1p,*s2p,*rp;
  
  bits = 2 * bitsize(__CAST(unsigned long,R_uma_char)) + 2 +
             bitsize(__CAST(unsigned long,MIN(na,nb))) + 2;
  ns1p = na * bits / GMP_NUMB_BITS + 1;
  ns2p = nb * bits / GMP_NUMB_BITS + 1;
  mem = __CAST(mp_limb_t*,
               ll_calloc((1 + 2 * (ns1p + ns2p))
                         * sizeof(mp_limb_t)));
  s1p = mem;
  s2p = &mem[ns1p];
  rp = &mem[ns1p + ns2p];
  ll_upol_mul_kronecker_modn_word_encode(s1p,a,na,bits);
  ll_upol_mul_kronecker_modn_word_encode(s2p,b,nb,bits);
  if ( ns1p > ns2p ) {
    mpn_mul(rp,s1p,ns1p,s2p,ns2p);
  }
  else {
    mpn_mul(rp,s2p,ns2p,s1p,ns1p);
  }
  nr = na + nb - 1;
  ll_upol_mul_kronecker_modn_word_decode(r,nr,rp,bits);
  ll_free(mem);
#else
  ll_upol_mul_naive(r,a,na,b,nb);
#endif
}

