/*
 * This file is part of decoding
 * Copyright 2013  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `void ll_upol_pow_mod(R *r,uma *nr,R *a,uma na,mpz_t e,R *m,\
uma nm);`
  Set `r` to `a(X) ^ e mod m` where `a` has size `na` and `m` has
  size `nm` and set `nr` to the size of `r`. Note that `e` must
  be a GMP integer. If `e` is of type `uma`, see the
  `ll_upol_pow_mod_uma` function instead.
*/
void ll_upol_pow_mod(R *r,uma *nr,R *a,uma na,mpz_t e,R *m,uma nm) {
#ifdef MY_UPOL_POW_MOD
  MY_UPOL_POW_MOD(r,nr,a,na,e,m,nm);
#else
  ll_upol_binpow_mod(r,nr,a,na,e,m,nm);
#endif
}

/**
> `void ll_upol_pow_mod_uma(R *r,uma *nr,R *a,uma na,uma e,R *m,\
uma nm);`
  Set `r` to `a(X) ^ e mod m` where `a` has size `na` and `m` has
  size `nm` and set `nr` to the size of `r`. Note that `e` must
  be a `uma` integer. If `e` is a multiprecision integer, see the
  `ll_upol_pow_mod` function instead.
*/
void ll_upol_pow_mod_uma(R *r,uma *nr,R *a,uma na,uma e,R *m,uma nm) {
#ifdef MY_UPOL_POW_MOD_UMA
  MY_UPOL_POW_MOD_UMA(r,nr,a,na,e,m,nm);
#else
  mpz_t gmp_e;
  mpz_init_set_si(gmp_e,__CAST(signed long,e));
  ll_upol_binpow_mod(r,nr,a,na,gmp_e,m,nm);
  mpz_clear(gmp_e);
#endif
}

