/*
 * This file is part of decoding
 * Copyright 2012-2013  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `void ll_upol_roots(R *r,uma *nr,R *a,uma na);`
  Set `r` of size `nr` to be the vector constituted by the roots
  of `a` of size `na`. By default no algorithm is provided and
  the function will terminate your program. Usually the ring
  provides its own root finding function, see
  `decoding-alphabets`(3) for details.
  You can provide your own root finding function with
  `MY_UPOL_ROOTS`, see `decoding-defines`(3) for details.
*/

void ll_upol_roots(R *r,uma *nr,R *a,uma na) {
#ifdef MY_UPOL_ROOTS
  MY_UPOL_ROOTS(r,nr,a,na);
#else
#ifdef RING_FF
  ll_upol_roots_fq(r,nr,a,na);
#else
  /* If no algorithm is provided, set nr = 0 */
  (void)r;
  nr[0] = 0;
  (void)a;
  (void)na;
#endif
#endif
}

