/*
 * This file is part of decoding
 * Copyright 2013  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/* root-finding, Algorithm 14.15 page 382,
   Gathen, Modern Computer Algebra, 2nd edition */

/**
> `void ll_upol_roots_fq(R *r,uma *nr,R *a,uma na);`
  Assuming that `R` is a finite field, set `r` to the `nr` roots in
  `R` of `a` of size `na`.
  Note that `a` *must* have degree `na - 1`.
*/

void ll_upol_roots_fq(R *r,uma *nr,R *a,uma na) {
#ifdef MY_UPOL_ROOTS_FQ
  MY_UPOL_ROOTS_FQ(r,nr,a,na);
#else
#ifdef RING_FF
  R x[2],*f,*h,*gcd;
  uma nf,nh,nh2,ngcd,ngcd2,nroots;
  struct ll_vec_lili *factors,*node;

  nf = ll_vec_true_size(a,na);
  if ( nf <= 1 ) {
    nr[0] = 0;
    return;
  }
  f = ll_vec_malloc_copy(a,nf);
  ll_upol_normalize(f,nf);
  R_inits(x[0],x[1],NULL);
  R_castint(x[1],1);
  nh2 = nf;
  h = ll_vec_malloc_init(nh2);
  ll_upol_pow_mod(h,&nh,x,2,R_card,f,nf);
  switch(nh) {
    case 1:
      R_castint(h[1],-1);
      nh = 2;
      break;
    case 2:
      R_sub(h[1],h[1],R_one);
      nh = ll_vec_true_size(h,nh);
      break;
    default:
      R_sub(h[1],h[1],R_one);
      break;
  }
  nr[0] = 0;
  if ( !ll_upol_iszero(h,nh) ) {
    ngcd2 = na;
    gcd = ll_vec_malloc_init(ngcd2);
    ll_upol_gcd(gcd,&ngcd,h,nh,f,nf);
    if ( ngcd > 1 ) {
      ll_upol_normalize(gcd,ngcd);
      factors = ll_upol_eqdegfact(gcd,ngcd,1);
      ll_vec_free(gcd,ngcd);
    }
    else {
      ll_vec_free(gcd,ngcd);
      goto ll_upol_roots_fq_free_fhx;
    }
  }
  else {
    factors = ll_upol_eqdegfact(f,nf,1);
  }
  nroots = 0;
  for( node = factors ; node ; node = node->next ) {
    R_div(r[nroots],node->a[0],node->a[1]);
    R_neg(r[nroots],r[nroots]);
    nroots++;
  }
  nr[0] = nroots;
  ll_vec_lili_list_free(factors);
  
ll_upol_roots_fq_free_fhx:
  ll_vec_free(h,nh2);
  R_clears(x[0],x[1],NULL);
  ll_vec_free(f,nf);
#else
  (void)r;
  (void)nr;
  (void)a;
  (void)na;
#endif
#endif
}

