/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `void ll_upol_shift_naive(R *r,R *a,uma na,R b);`
  Set `r` to `a(X + b)` where `a` has size `na`.
  The schoolbook algorithm is used.
*/

void ll_upol_shift_naive(R *r,R *a,uma na,R b) {
  uma i,j;
  R_copy(r[0],a[na - 1]);
  for( i = na - 2,j = 1 ; i >= 0 ; i--,j++ ) {
    ll_upol_mul_by_affine(r,r,j,b);
    R_add(r[0],r[0],a[i]);
  }
}

/**
> `void ll_upol_shift(R *r,R *a,uma na,R b);`
  Set `r` to `a(X + b)` where `a` has size `na`.
  By default, if the ring has caracteristic 2 (`RING_CHAR2`)
  `ll_upol_shift_car2` is called, and `ll_upol_shift_fast` is called
  otherwise. In both cases the dichotomy algorithm is used.
  You can also provide your own shifting function,
  see `decoding-defines`(3) for details.
*/

void ll_upol_shift(R *r,R *a,uma na,R b) {
#ifdef MY_UPOL_SHIFT
  MY_UPOL_SHIFT(r,a,na,b);
#else
#ifdef RING_CHAR2
  ll_upol_shift_car2(r,a,na,b);
#else
  ll_upol_shift_fast(r,a,na,b);
#endif
#endif
}

