/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/* NODE MANIPULATION */

/**
> `struct ll_vec_lili *ll_vec_lili_node_malloc(uma na);`
  Return a `ll_vec_lili` struct containing an allocated vector of
  size `na`.
*/

struct ll_vec_lili *ll_vec_lili_node_malloc(uma na) {
  struct ll_vec_lili *node;
  node = __CAST(struct ll_vec_lili*,
                ll_malloc(sizeof(struct ll_vec_lili)));
  node->next = NULL;
  node->prev = NULL;
  node->na = na;
  node->a = ll_vec_malloc_init(na);
  return node;
}

/**
> `struct ll_vec_lili *ll_vec_lili_node_malloc_copy(R *a,uma na);`
  Return a `ll_vec_lili` struct containing a deep copy of the vector
  `a` of size `na`.
*/

struct ll_vec_lili *ll_vec_lili_node_malloc_copy(R *a,uma na) {
  struct ll_vec_lili *node;
  node = __CAST(struct ll_vec_lili*,
                ll_malloc(sizeof(struct ll_vec_lili)));
  node->next = NULL;
  node->prev = NULL;
  node->na = na;
  node->a = ll_vec_malloc_copy(a,na);
  return node;
}

/**
> `void ll_vec_lili_node_free(struct ll_vec_lili *node);`
  Free the `ll_vec_lili` struct `node`.
*/

void ll_vec_lili_node_free(struct ll_vec_lili *node) {
  ll_vec_free(node->a,node->na);
  ll_free(node);
}

/* LINKED LIST MANIPULATION */

/**
> `void ll_vec_lili_node_attach_after(struct ll_vec_lili *after,\
  struct ll_vec_lili *node);`
  Attach the `node` after `after`.
*/

void ll_vec_lili_node_attach_after(struct ll_vec_lili *after,
                                   struct ll_vec_lili *node)
{
  struct ll_vec_lili *tmp;
  tmp = after->next;
  after->next = node;
  node->prev = after;
  node->next = tmp;
}

/**
> `void ll_vec_lili_node_attach_before(struct ll_vec_lili *before,\
  struct ll_vec_lili *node);`
  Attach `node` before `before`.
*/

void ll_vec_lili_node_attach_before(struct ll_vec_lili *before,
                                    struct ll_vec_lili *node)
{
  struct ll_vec_lili *tmp;
  tmp = before->prev;
  before->prev = node;
  node->next = before;
  node->prev = tmp;
}

/**
> `void ll_vec_lili_node_detach(struct ll_vec_lili *node);`
  Detach `node`.
*/

void ll_vec_lili_node_detach(struct ll_vec_lili *node) {
  if ( node->next ) node->next->prev = node->prev;
  if ( node->prev ) node->prev->next = node->next;
}

/**
> `void ll_vec_lili_list_free(struct ll_vec_lili *start);`
  Free the linked list whose first node is `start`.
*/

void ll_vec_lili_list_free(struct ll_vec_lili *start) {
  struct ll_vec_lili *node,*tmp;
  for( node = start ; node ; ) {
    tmp = node->next;
    ll_vec_lili_node_free(node);
    node = tmp;
  }
}

/**
> `int ll_vec_lili_list_in(struct ll_vec_lili *start,R *a,uma na);`
  Return nonzero if and only if `a` of size `na` is within
  the linked list pointed to by `start`.
*/

int ll_vec_lili_list_in(struct ll_vec_lili *start,R *a,uma na) {
  struct ll_vec_lili *node;
  for( node = start ; node ; node = node->next ) {
    if ( node->na == na && !ll_vec_cmp(node->a,a,na) ) return -1;
  }
  return 0;
}

