/*
 * This file is part of decoding
 * Copyright 2012-2013  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/* low-level structs */
struct ll_upol_shift_fast_pc { /* pc is for PreComputations */
  R **pow;
  uma npow;
};

struct ll_upol_shift_car2_pc {
  R *pow;
  uma npow;
};

struct ll_dbpol_shift_fast_pc {
  struct ll_upol_shift_fast_pc *powX,*powY;
};

struct ll_dbpol_shift_car2_pc {
  struct ll_upol_shift_car2_pc *powX,*powY;
};

struct ll_vec_lili {
  R *a;
  uma na;
  struct ll_vec_lili *next,*prev;
};

struct ll_upol_prod_tree {
  R *a;
  uma na;
  struct ll_upol_prod_tree *left,*right;
};

/* low level functions */

/* lowlevels/array.c */
void ll_array_add(R*,R*,uma,R*,uma);
void ll_array_sub(R*,R*,uma,R*,uma);
void ll_array_add_mul_by_cte(R*,uma,R*,uma,R);

/* lowlevels/dbpol_eval_horner.c */
void ll_dbpol_eval_horner(R,R*,uma,uma,R,R);
void ll_dbpol_eval_horner_upol(R*,uma*,R*,uma,uma,R*,uma);

/* lowlevels/dbpol_mul_by_affine.c */
void ll_dbpol_mul_by_affine(R*,R*,uma,uma,R);

/* lowlevels/dbpol_mul_by_upol.c */
void ll_dbpol_mul_by_upolX(R*,R*,uma,uma,R*,uma);
void ll_dbpol_mul_by_upolY(R*,R*,uma,uma,R*,uma);

/* lowlevels/dbpol_mul_by_Y.c */
void ll_dbpol_mul_by_Y(R*,R*,uma,uma);
void ll_dbpol_mul_by_Yn(R*,R*,uma,uma,uma);

/* lowlevels/dbpol_mul.c */
void ll_dbpol_mul_naive(R*,R*,uma,uma,R*,uma,uma);
void ll_dbpol_mul(R*,R*,uma,uma,R*,uma,uma);

/* lowlevels/dbpol_print.c */
void ll_dbpol_print_factorized(FILE*,R*,uma,uma);

/* lowlevels/dbpol_replace_X_by_XY.c */
void ll_dbpol_replace_X_by_XYn(R*,uma*,R*,uma,uma,uma);

/* lowlevels/dbpol_shift.c */
void ll_dbpol_shift_naive(R*,R*,uma,uma,R,R);
void ll_dbpol_shift(R*,R*,uma,uma,R,R);

/* lowlevels/dbpol_shift_fast.c */
void ll_dbpol_shift_fast_rec(R*,R*,uma,uma,R,R**,R*);
void ll_dbpol_shift_fast2(R*,R*,uma,uma,R,R,
                          struct ll_dbpol_shift_fast_pc*);
struct ll_dbpol_shift_fast_pc *ll_dbpol_shift_fast_prepare(uma,uma,R,R);
void ll_dbpol_shift_fast(R*,R*,uma,uma,R,R);
void ll_dbpol_shift_fast_finish(struct ll_dbpol_shift_fast_pc*);
void ll_dbpol_shift_car2_rec(R*,uma,uma,R,R*);
void ll_dbpol_shift_car22(R*,R*,uma,uma,R,R,
                          struct ll_dbpol_shift_car2_pc*);
struct ll_dbpol_shift_car2_pc *ll_dbpol_shift_car2_prepare(uma,uma,R,R);
void ll_dbpol_shift_car2(R*,R*,uma,uma,R,R);
void ll_dbpol_shift_car2_finish(struct ll_dbpol_shift_car2_pc*);

/* lowlevels/dbpol_shift_in_X.c */
void ll_dbpol_shift_in_X(R*,R*,uma,uma,R);

/* lowlevels/dbpol_Xroots.c */
struct ll_vec_lili *ll_dbpol_Xroots_rec(R*,uma,uma,uma,uma,uma);
struct ll_vec_lili *ll_dbpol_Xroots(R*,uma,uma,uma);

/* lowlevels/koetter.c */
uma ll_koetter_single(R*,uma*,uma*,R*,uma,uma,uma,uma,R*,uma);
uma ll_koetter_multi(R*,uma*,uma*,R*,uma,uma,uma,uma*,R*,uma);

/* lowlevels/koetter_shift.c */
uma ll_koetter_shift_single(R*,uma*,uma*,R*,uma,uma,uma,uma,R*,uma);
uma ll_koetter_shift_multi(R*,uma*,uma*,R*,uma,uma,uma,uma*,R*,uma);

/* lowlevels/koetter_shift_omp.c */
uma ll_koetter_shift_omp_single(R*,uma*,uma*,R*,uma,uma,uma,uma,R*,uma);
uma ll_koetter_shift_omp_multi(R*,uma*,uma*,R*,uma,uma,uma,uma*,R*,uma);

/* lowlevels/koetter_sudan.c */
uma ll_koetter_sudan(R*,uma*,uma*,R*,uma,uma,uma,R*,uma);

/* lowlevels/koetter_sudan_omp.c */
uma ll_koetter_sudan_omp(R*,uma*,uma*,R*,uma,uma,uma,R*,uma);

/* lowlevels/linked_list_vec.c */
struct ll_vec_lili *ll_vec_lili_node_malloc(uma);
struct ll_vec_lili *ll_vec_lili_node_malloc_copy(R*,uma);
void ll_vec_lili_node_free(struct ll_vec_lili*);
void ll_vec_lili_node_attach_after(struct ll_vec_lili*,
                                   struct ll_vec_lili*);
void ll_vec_lili_node_attach_before(struct ll_vec_lili*,
                                    struct ll_vec_lili*);
void ll_vec_lili_node_detach(struct ll_vec_lili*);
void ll_vec_lili_list_free(struct ll_vec_lili*);
int ll_vec_lili_list_in(struct ll_vec_lili*,R*,uma);

/* lowlevels/mat.c */
void ll_mat_copy(R*,R*,uma,uma);
R *ll_mat_malloc_init(uma,uma);
R *ll_mat_malloc_random(uma,uma);
void ll_mat_free(R*,uma,uma);
void ll_mat_transpose(R*,R*,uma,uma);
void ll_mat_print(FILE*,R*,uma,uma);
void ll_mat_put(R*,uma,uma,uma,R*,uma,uma);

/* lowlevels/print.c */
void ll_print(FILE*,const char*,...);

/* lowlevels/upol_eucldiv.c */
void ll_upol_quorem_naive(R*,uma*,R*,R*,uma,R*,uma);
void ll_upol_quo_naive(R*,R*,uma,R*,uma);
void ll_upol_rem_naive(R*,uma*,R*,uma,R*,uma);
void ll_upol_quorem(R*,uma*,R*,R*,uma,R*,uma);
void ll_upol_quo(R*,R*,uma,R*,uma);
void ll_upol_rem(R*,uma*,R*,uma,R*,uma);

/* lowlevels/upol_gcd.c */
void ll_upol_gcd_naive(R*,uma*,R*,uma,R*,uma);
void ll_upol_gcd(R*,uma*,R*,uma,R*,uma);

/* lowlevels/upol_eval_horner.c */
void ll_upol_eval_horner(R,R*,uma,R);
void ll_upol_multi_eval_horner(R*,R*,uma,R*,uma);

/* lowlevels/upol_mul_by_affine.c */
void ll_upol_mul_by_affine(R*,R*,uma,R);

/* lowlevels/upol_mul_by_X.c */
void ll_upol_mul_by_X(R*,R*,uma);
void ll_upol_mul_by_Xn(R*,R*,uma,uma);

/* lowlevels/upol_mul.c */
void ll_upol_mul_naive(R*,R*,uma,R*,uma);
void ll_upol_mul(R*,R*,uma,R*,uma);

/* lowlevels/upol_normalize.c */
void ll_upol_normalize(R*,uma);

/* lowlevels/upol_print.c */
void ll_upol_print(FILE*,R*,uma);

/* lowlevels/upol_random.c */
void ll_upol_random(R*,uma);
R *ll_upol_malloc_random(uma);

/* lowlevels/upol_roots.c */
void ll_upol_roots(R*,uma*,R*,uma);

/* lowlevels/upol_shift.c */
void ll_upol_shift_naive(R*,R*,uma,R);
void ll_upol_shift(R*,R*,uma,R);

/* lowlevels/upol_shift_fast.c */
void ll_upol_shift_fast_rec(R*,R*,uma,R,R**,R*);
void ll_upol_shift_fast2(R*,R*,uma,R,struct ll_upol_shift_fast_pc*);
struct ll_upol_shift_fast_pc *ll_upol_shift_fast_prepare(uma,R);
void ll_upol_shift_fast_finish(struct ll_upol_shift_fast_pc*);
void ll_upol_shift_fast(R*,R*,uma,R);
void ll_upol_shift_car2_rec(R*,R*,uma,R,R*);
void ll_upol_shift_car22(R*,R*,uma,R,struct ll_upol_shift_car2_pc*);
struct ll_upol_shift_car2_pc *ll_upol_shift_car2_prepare(uma,R);
void ll_upol_shift_car2_finish(struct ll_upol_shift_car2_pc*);
void ll_upol_shift_car2(R*,R*,uma,R);

/* lowlevels/upol_square.c */
void ll_upol_square_naive(R*,R*,uma);
void ll_upol_square(R*,R*,uma);

/* lowlevels/upol_kronecker_modn_word.c */
void ll_put_word_into_unsigned_long(unsigned long*,R,uma);
void ll_get_word_from_unsigned_long_1(R,unsigned long*,uma,
                                      unsigned long);
void ll_upol_mul_kronecker_modn_word_encode(mp_limb_t*,R*,uma,uma);
void ll_upol_mul_kronecker_modn_word_decode(R*,uma,mp_limb_t*,uma);
void ll_upol_mul_kronecker_modn_word(R*,R*,uma,R*,uma);

/* lowlevels/utils.c */
void ll_die(const char*);

/* lowlevels/vec.c */
void ll_vec_add(R*,R*,R*,uma);
void ll_vec_neg(R*,R*,uma);
void ll_vec_sub(R*,R*,R*,uma);
void ll_vec_sub_sub(R*,R*,R*,uma);
void ll_vec_add_mul(R*,R*,R*,uma);
void ll_vec_mul_by_cte(R*,R*,uma,R);
void ll_vec_add_mul_by_cte(R*,R*,uma,R);
void ll_vec_sub_mul(R*,R*,R*,uma);
void ll_vec_init(R*,uma);
void ll_vec_init_random(R*,uma);
void ll_vec_clear(R*,uma);
uma ll_vec_hamming_weight(R*,uma);
uma ll_vec_hamming_distance(R*,R*,uma);
void ll_vec_zero(R*,uma);
uma ll_vec_iszero(R*,uma);
void ll_vec_random(R*,uma);
void ll_vec_random_hamming_weight(R*,uma,uma);
void ll_vec_copy(R*,R*,uma);
uma ll_vec_true_size(R*,uma);
R *ll_vec_malloc_init(uma);
R *ll_vec_realloc(R*,uma,uma);
R *ll_vec_malloc_copy(R*,uma);
R *ll_vec_malloc_random(uma);
void ll_vec_free(R*,uma);
uma ll_vec_cmp(R*,R*,uma);
void ll_vec_print(FILE*,R*,uma);

/* lowlevels/upol_prod_affine.c */
void ll_upol_prod_affine(R*,R*,uma);
void ll_upol_prod_affine_neg(R*,R*,uma);

/* lowlevels/dbpol_upol_map_XtoY.c */
void ll_dbpol_upol_map_XtoY(R*,uma,R*,uma);

/* lowlevels/binomial.c */
void ll_binomial(R,uma,uma);

/* lowlevels/upol_discrepancy.c */
void ll_upol_discrepancy(R,R*,uma,uma,R);

/* lowlevels/dbpol_discrepancy.c */
void ll_dbpol_discrepancy(R,R*,uma,uma,uma,uma,R,R);

/* lowlevels/koetter_binomial.c */
uma ll_koetter_binomial_single(R*,uma*,uma*,R*,uma,uma,uma,uma,R*,uma);

/* lowlevels/upol_eval.c */
void ll_upol_eval(R,R*,uma,R);
void ll_upol_multi_eval(R*,R*,uma,R*,uma);

/* lowlevels/dbpol_eval.c */
void ll_dbpol_eval(R,R*,uma,uma,R,R);
void ll_dbpol_eval_upol(R*,uma*,R*,uma,uma,R*,uma);

/* lowlevels/upol_roots_fq.c */
void ll_upol_roots_fq(R*,uma*,R*,uma);

/* lowlevels/upol_pow_mod.c */
void ll_upol_pow_mod(R*,uma*,R*,uma,mpz_t,R*,uma);
void ll_upol_pow_mod_uma(R*,uma*,R*,uma,uma,R*,uma);

/* lowlevels/upol_binpow_mod.c */
void ll_upol_binpow_mod(R*,uma*,R*,uma,mpz_t,R*,uma);

/* lowlevels/upol_derivative.c */
void ll_upol_derivative_naive(R*,R*,uma);
void ll_upol_derivative(R*,R*,uma);

/* lowlevels/upol_eqdegsplit.c */
void ll_upol_trace_poly(R*,uma*,R*,uma,uma,R*,uma);
void ll_upol_cantor_zassenhaus(R*,uma*,R*,uma,uma);
void ll_upol_eqdegsplit(R*,uma*,R*,uma,uma);

/* lowlevels/upol_eqdegfact.c */
struct ll_vec_lili *ll_upol_eqdegfact(R*,uma,uma);

/* decoding/lowlevels/upol.c */
int ll_upol_iszero(R*,uma);

/* decoding/lowlevels/upol_eea.c */
void ll_upol_eea_naive(R*,uma*,R*,uma*,R*,uma*,R*,uma,R*,uma);
void ll_upol_eea(R*,uma*,R*,uma*,R*,uma*,R*,uma,R*,uma);

/* decoding/lowlevels/upol_partial_eea.c */
void ll_upol_partial_eea_naive(R*,uma*,R*,uma*,R*,uma*,R*,uma,R*,uma,uma);
void ll_upol_partial_eea(R*,uma*,R*,uma*,R*,uma*,R*,uma,R*,uma,uma);

/* decoding/lowlevels/vec_prod.c */
void ll_vec_prod_naive(R,R*,uma);
void ll_vec_prod(R,R*,uma);

/* decoding/lowlevels/upol_interpolation.c */
void ll_upol_interpolation_naive(R*,uma*,R*,R*,uma);
void ll_upol_interpolation(R*,uma*,R*,R*,uma);

/* decoding/lowlevels/upol_mul_kronecker_char2_word.c */
void ll_put_word_into_unsigned_long(unsigned long*,R,uma);
void ll_get_word_from_unsigned_long_1(R,unsigned long*,uma,
                                      unsigned long);
void ll_upol_mul_kronecker_char2_word_encode(unsigned long*,
                                             R*,uma,uma);
void ll_upol_mul_kronecker_char2_word_decode(R*,uma,unsigned long*,
                                             uma);
void ll_upol_mul_kronecker_char2_word(R*,R*,uma,R*,uma);

/* decoding/lowlevels/upol_div_by_affine.c */
void ll_upol_div_by_affine(R*,R*,uma,R);

