/*
 * This file is part of decoding
 * Copyright 2012-2013  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

void test_print_r(void) {
  fputs("testing dense bivariate polynomials over " __S(R) "\n",stdout);
}

void test_mul_naive(uma sz,uma n,uma comm) {
  uma i,j,k;
  R *a,*b,*c,*ab,*ba,*bc,*ac,*abc1,*abc2,*one;
  fputs("  naive multiplication...",stdout);
  fflush(stdout);
  a = ll_vec_malloc_init(sz * sz);
  b = ll_vec_malloc_init(sz * sz);
  c = ll_vec_malloc_init(sz * sz);
  ab = ll_vec_malloc_init(4 * sz * sz);
  ba = ll_vec_malloc_init(4 * sz * sz);
  bc = ll_vec_malloc_init(4 * sz * sz);
  ac = ll_vec_malloc_init(4 * sz * sz);
  abc1 = ll_vec_malloc_init(9 * sz * sz);
  abc2 = ll_vec_malloc_init(9 * sz * sz);
  one = ll_vec_malloc_init(sz * sz);
  R_castint(one[0],1);
  for( j = 1 ; j < sz ; j++ ) {
    for( i = 0 ; i < n ; i++ ) {
      ll_vec_random(a,j * j);
      ll_vec_random(b,j * j);
      ll_vec_random(c,j * j);
      if ( comm ) {
        ll_dbpol_mul_naive(ab,a,j,j,b,j,j);
        ll_dbpol_mul_naive(ba,b,j,j,a,j,j);
        if ( ll_vec_cmp(ab,ba,(2 * j - 1) * (2 * j - 1)) ) {
          ll_print(stdout,"FAIL\n\na * b != b * a (%d)\n"
                          "a = %q\n\nb = %q\n",j,a,j,j,b,j,j);
          exit(-1);
        }
      }
      ll_dbpol_mul_naive(ab,a,j,j,b,j,j);
      ll_dbpol_mul_naive(bc,b,j,j,c,j,j);
      ll_dbpol_mul_naive(abc1,ab,2 * j - 1,2 * j - 1,c,j,j);
      ll_dbpol_mul_naive(abc2,a,j,j,bc,2 * j - 1,2 * j - 1);
      if ( ll_vec_cmp(abc1,abc2,(3 * j - 2) * (3 * j - 2)) ) {
        ll_print(stdout,"FAIL\n\n(a * b) * c != a * (b * c)\n"
                        "a = %q\n\nb = %q\n\nc = %q\n",
                        a,j,j,b,j,j,c,j,j);
        exit(-1);
      }
      ll_vec_add(ab,a,b,j * j);
      ll_dbpol_mul_naive(abc1,ab,j,j,c,j,j);
      ll_dbpol_mul_naive(ac,a,j,j,c,j,j);
      ll_dbpol_mul_naive(bc,b,j,j,c,j,j);
      ll_vec_add(abc2,ac,bc,(2 * j - 1) * (2 * j -1));
      if ( ll_vec_cmp(abc1,abc2,(2 * j - 1) * (2 * j - 1)) ) {
        ll_print(stdout,"FAIL\n\n(a + b) * c != (a * c) + (b * c)\n"
                        "a = %q\n\nb = %q\n\nc = %q\n",
                        a,j,j,b,j,j,c,j,j);
        exit(-1);
      }
      ll_dbpol_mul_naive(ab,one,j,j,a,j,j);
      for( k = 0 ; k < j ; k++ ) {
        if ( ll_vec_cmp(&ab[k * (2 * j - 1)],&a[k * j],j) ||
             !ll_vec_iszero(&ab[k * (2 * j - 1) + j],j - 1) ) {
          ll_print(stdout,"FAIL\n\na * 1 != a\na = %q\n",a,j,j);
          ll_print(stdout,"one = %q\n\n",one,j,j);
          ll_print(stdout,"ab = %q\n\n",ab,j,j);
          exit(-1);
        }
      }
    }
  }
  ll_vec_free(a,sz * sz);
  ll_vec_free(b,sz * sz);
  ll_vec_free(c,sz * sz);
  ll_vec_free(ab,4 * sz * sz);
  ll_vec_free(ba,4 * sz * sz);
  ll_vec_free(bc,4 * sz * sz);
  ll_vec_free(ac,4 * sz * sz);
  ll_vec_free(abc1,9 * sz * sz);
  ll_vec_free(abc2,9 * sz * sz);
  ll_vec_free(one,sz * sz);
  fputs("OK\n",stdout);
}

void test_mul(uma sz,uma n) {
  uma i,j;
  R *a,*b,*ab1,*ab2;
  fputs("  multiplication...",stdout);
  fflush(stdout);
  a = ll_vec_malloc_init(sz * sz);
  b = ll_vec_malloc_init(sz * sz);
  ab1 = ll_vec_malloc_init(4 * sz * sz);
  ab2 = ll_vec_malloc_init(4 * sz * sz);
  for( j = 1 ; j < sz ; j++ ) {
    for( i = 0 ; i < n ; i++ ) {
      ll_vec_random(a,j * j);
      ll_vec_random(b,j * j);
      ll_dbpol_mul_naive(ab1,a,j,j,b,j,j);
      ll_dbpol_mul(ab2,a,j,j,b,j,j);
      if ( ll_vec_cmp(ab1,ab2,(2 * j - 1) * (2 * j -1)) ) {
        ll_print(stdout,"FAIL:\n\na * b != a * b\n\n"
                        "a = %q\n\nb = %q\n",a,j,j,b,j,j);
        exit(-1);
      }
    }
  }
  ll_vec_free(a,sz * sz);
  ll_vec_free(b,sz * sz);
  ll_vec_free(ab1,4 * sz * sz);
  ll_vec_free(ab2,4 * sz * sz);
  fputs("OK\n",stdout);
}

void test_shift_naive(uma sz,uma n,uma comm) {
  uma i,j;
  R a1,a2,b,*f1,*f2,*f3,*g1,*g2,*g3,*g4;
  fputs("  naive shift...",stdout);
  R_inits(a1,a2,b,NULL);
  f1 = ll_vec_malloc_init(sz * sz);
  f2 = ll_vec_malloc_init(sz * sz);
  f3 = ll_vec_malloc_init(4 * sz * sz);
  g1 = ll_vec_malloc_init(4 * sz * sz);
  g2 = ll_vec_malloc_init(4 * sz * sz);
  g3 = ll_vec_malloc_init(4 * sz * sz);
  g4 = ll_vec_malloc_init(4 * sz * sz);
  for( j = 1 ; j < sz ; j++ ) {
    for( i = 0 ; i < n ; i++ ) {
      R_random(a1);
      R_random(a2);
      R_random(b);
      ll_vec_random(f1,j * j);
      ll_dbpol_shift_naive(g1,f1,j,j,a1,a2);
      ll_vec_mul_by_cte(g1,g1,j * j,b);
      ll_vec_mul_by_cte(f1,f1,j * j,b);
      ll_dbpol_shift_naive(g2,f1,j,j,a1,a2);
      if ( ll_vec_cmp(g1,g2,j * j) ) {
        ll_print(stdout,"FAIL\n\n(b f1)(X+a1,Y+a2) != "
                        "b f1(X+a1,Y+a2)\n"
                        "a1 = %e\na2 = %e\nb = %e\nf1 = %q\n",
                        a1,a2,b,f1,j,j);
        exit(-1);
      }
      ll_vec_random(f1,j * j);
      ll_vec_random(f2,j * j);
      ll_vec_add(f3,f1,f2,j * j);
      ll_dbpol_shift_naive(g3,f3,j,j,a1,a2);
      ll_dbpol_shift_naive(g1,f1,j,j,a1,a2);
      ll_dbpol_shift_naive(g2,f2,j,j,a1,a2);
      ll_vec_add(g1,g1,g2,j * j);
      if ( ll_vec_cmp(g1,g3,j * j) ) {
        ll_print(stdout,"FAIL\n\n(f1+f2)(X+a1,Y+a2) != "
                        "f1(X+a1,Y+a2) + f2(X+a1,Y+a2)\n"
                        "a1 = %e\na2 = %e\nf1 = %q\n\nf2 = %q\n",
                        a1,a2,f1,j,j,f2,j,j);
        exit(-1);
      }
      if ( comm ) {
        ll_dbpol_mul(f3,f1,j,j,f2,j,j);
        ll_dbpol_shift_naive(g3,f3,2 * j - 1,2 * j - 1,a1,a2);
        ll_dbpol_shift_naive(g1,f1,j,j,a1,a2);
        ll_dbpol_shift_naive(g2,f2,j,j,a1,a2);
        ll_dbpol_mul(g4,g1,j,j,g2,j,j);
        if ( ll_vec_cmp(g3,g4,(2 * j - 1) * (2 * j - 1)) ) {
          ll_print(stdout,"FAIL\n\n(f1 * f2)(X+a1,Y+a2) != "
                          "f1(X+a1,Y+a2) * f2(X+a1,Y+a2)\n"
                          "j = %d\na1 = %e\na2 = %e\nf1 = %q\n\nf2 = %q\n",
                          j,a1,a2,f1,j,j,f2,j,j);
          exit(-1);
        }        
      }
    }
  }
  R_clears(a1,a2,b,NULL);
  ll_vec_free(f1,sz * sz);
  ll_vec_free(f2,sz * sz);
  ll_vec_free(f3,4 * sz * sz);
  ll_vec_free(g1,4 * sz * sz);
  ll_vec_free(g2,4 * sz * sz);
  ll_vec_free(g3,4 * sz * sz);
  ll_vec_free(g4,4 * sz * sz);
  fputs("OK\n",stdout);
}

void test_shift(uma sz,uma n) {
  uma i,j;
  R a1,a2,*f,*g1,*g2;
  fputs("  shift...",stdout);
  R_inits(a1,a2,NULL);
  f = ll_vec_malloc_init(sz * sz);
  g1 = ll_vec_malloc_init(sz * sz);
  g2 = ll_vec_malloc_init(sz * sz);
  for( j = 1 ; j < sz ; j++ ) {
    for( i = 0 ; i < n ; i++ ) {
      R_random(a1);
      R_random(a2);
      ll_vec_random(f,j * j);
      ll_dbpol_shift_naive(g1,f,j,j,a1,a2);
      ll_dbpol_shift(g2,f,j,j,a1,a2);
      if ( ll_vec_cmp(g1,g2,j * j) ) {
        ll_print(stdout,"FAIL\n\n"
                        "f(X + a1,Y + a2) != f(X + a1,Y + a2)\n"
                        "a1 = %e\na2 = %e\n\nf = %q\n",
                        a1,a2,f,j,j);
        exit(-1);
      }
    }
  }
  R_clears(a1,a2,NULL);
  ll_vec_free(f,sz * sz);
  ll_vec_free(g1,sz * sz);
  ll_vec_free(g2,sz * sz);
  fputs("OK\n",stdout);
}

void test_eval_horner(uma sz,uma n,uma comm) {
  uma i,j;
  R *f1,*f2,*f3,a1,a2,b,c1,c2;
  fputs("  Horner evaluation...",stdout);
  fflush(stdout);
  R_inits(a1,a2,b,c1,c2,NULL);
  f1 = ll_vec_malloc_init(sz * sz);
  f2 = ll_vec_malloc_init(sz * sz);
  f3 = ll_vec_malloc_init(4 * sz * sz);
  for( j = 1 ; j < sz; j++ ) {
    for( i = 0 ; i < n ; i++ ) {
      fflush(stdout);
      R_random(a1);
      R_random(a2);
      R_random(b);
      ll_vec_random(f1,j * j);
      ll_dbpol_eval_horner(c1,f1,j,j,a1,a2);
      R_mul(c1,b,c1);
      ll_vec_mul_by_cte(f2,f1,j * j,b);
      ll_dbpol_eval_horner(c2,f2,j,j,a1,a2);
      if ( R_cmp(c1,c2) ) {
        ll_print(stdout,"FAIL:\n\n(b f1)(a1,a2) != b f1(a1,a2)\n"
                        "a1 = %e\na2 = %e\nb = %e\nf1 = %q\n",
                        a1,a2,b,f1,j,j);
        exit(-1);
      }
      ll_vec_random(f2,j * j);
      ll_dbpol_eval_horner(c1,f1,j,j,a1,a2);
      ll_dbpol_eval_horner(c2,f2,j,j,a1,a2);
      R_add(c1,c1,c2);
      ll_vec_add(f3,f1,f2,j * j);
      ll_dbpol_eval_horner(c2,f3,j,j,a1,a2);
      if ( R_cmp(c1,c2) ) {
        ll_print(stdout,"FAIL:\n\n(f1 + f2)(a1,a2) != "
                        "f1(a1,a2) + f2(a1,a2)\n"
                        "a1 = %e\na2 = %e\nf1 = %q\nf2 = %q\n",
                        a1,a2,f1,j,j,f2,j,j);
        exit(-1);
      }
      if ( comm ) {
        ll_dbpol_eval_horner(c1,f1,j,j,a1,a2);
        ll_dbpol_eval_horner(c2,f2,j,j,a1,a2);
        R_mul(c1,c1,c2);
        ll_dbpol_mul(f3,f1,j,j,f2,j,j);
        ll_dbpol_eval_horner(c2,f3,2 * j - 1,2 * j - 1,a1,a2);
        if ( R_cmp(c1,c2) ) {
          ll_print(stdout,"FAIL:\n\n(f1 * f2)(a1,a2) != "
                          "f1(a1,a2) * f2(a1,a2)\n"
                          "a1 = %e\na2 = %e\nf1 = %q\nf2 = %q\n",
                          a1,a2,f1,j,j,f2,j,j);
          exit(-1);
        }
      }
    }
  }
  R_clears(a1,a2,b,c1,c2,NULL);
  ll_vec_free(f1,sz * sz);
  ll_vec_free(f2,sz * sz);
  ll_vec_free(f3,4 * sz * sz);
  fputs("OK\n",stdout);
}

void test_eval_horner_upol(uma sz,uma n,uma comm) {
  uma i,j;
  R *f1,*f2,*f3,*a,*b,*c1,*c2,*c3;
  uma nc1,nc2,nc3;
  fputs("  Horner evaluation of upol...",stdout);
  fflush(stdout);
  a = ll_vec_malloc_init(sz);
  b = ll_vec_malloc_init(sz);
  f1 = ll_vec_malloc_init(sz * sz);
  f2 = ll_vec_malloc_init(sz * sz);
  f3 = ll_vec_malloc_init(4 * sz * sz);
  c1 = ll_vec_malloc_init(4 * sz * sz + 2 * sz);
  c2 = ll_vec_malloc_init(4 * sz * sz + 2 * sz);
  c3 = ll_vec_malloc_init(4 * sz * sz + 2 * sz);
  for( j = 1 ; j < sz; j++ ) {
    for( i = 0 ; i < n ; i++ ) {
      ll_vec_random(a,j);
      ll_vec_random(b,j);
      ll_vec_random(f1,j * j);
      ll_dbpol_eval_horner_upol(c2,&nc1,f1,j,j,a,j);
      ll_upol_mul(c1,c2,nc1,b,j);
      ll_dbpol_mul_by_upolX(f3,f1,j,j,b,j);
      ll_dbpol_eval_horner_upol(c2,&nc2,f3,2 * j - 1,j,a,j);
      nc1 = ll_vec_true_size(c1,nc1 + j - 1);
      nc2 = ll_vec_true_size(c2,nc2);
      if ( nc1 != nc2 || ll_vec_cmp(c1,c2,nc2) ) {
        ll_print(stdout,"FAIL:\n\n(b f1)(a) != b f1(a)\n"
                        "a = %p\n\nb = %p\n\nf1 = %q\n",
                        a,j,b,j,f1,j,j);
        exit(-1);
      }
      ll_vec_random(f2,j * j);
      ll_dbpol_eval_horner_upol(c1,&nc1,f1,j,j,a,j);
      ll_dbpol_eval_horner_upol(c2,&nc2,f2,j,j,a,j);
      ll_array_add(c1,c1,nc1,c2,nc2);
      nc1 = MAX(nc1,nc2);
      ll_vec_add(f3,f1,f2,j * j);
      ll_dbpol_eval_horner_upol(c2,&nc2,f3,j,j,a,j);
      nc1 = ll_vec_true_size(c1,nc1);
      nc2 = ll_vec_true_size(c2,nc2);
      if ( nc1 != nc2 || ll_vec_cmp(c1,c2,nc1) ) {
        ll_print(stdout,"FAIL:\n\n(f1 + f2)(a) != f1(a) + f2(a)\n"
                        "a = %p\n\nf1 = %q\n\nf2 = %q\n",
                        a,j,f1,j,j,f2,j,j);
        ll_print(stdout,"c1 = %p\n\nc2 = %p\n",c1,nc1,c2,nc2);
        exit(-1);
      }
      if ( comm ) {
        ll_dbpol_eval_horner_upol(c1,&nc1,f1,j,j,a,j);
        ll_dbpol_eval_horner_upol(c2,&nc2,f2,j,j,a,j);
        ll_upol_mul(c3,c1,nc1,c2,nc2);
        nc3 = ll_vec_true_size(c3,nc1 + nc2 - 1);
        ll_dbpol_mul(f3,f1,j,j,f2,j,j);
        ll_dbpol_eval_horner_upol(c2,&nc2,f3,2 * j - 1,2 * j - 1,a,j);
        nc2 = ll_vec_true_size(c2,nc2);
        if ( nc2 != nc3 || ll_vec_cmp(c2,c3,nc3) ) {
          ll_print(stdout,"FAIL:\n\n(f1 * f2)(a) != "
                          "f1(a) * f2(a)\n"
                          "a = %p\n\nf1 = %q\n\nf2 = %q\n",
                          a,j,f1,j,j,f2,j,j);
          exit(-1);
        }
      }
    }
  }
  ll_vec_free(a,sz);
  ll_vec_free(b,sz);
  ll_vec_free(f1,sz * sz);
  ll_vec_free(f2,sz * sz);
  ll_vec_free(f3,4 * sz * sz);
  ll_vec_free(c1,2 * sz * sz + 2 * sz);
  ll_vec_free(c2,2 * sz * sz + 2 * sz);
  ll_vec_free(c3,2 * sz * sz + 2 * sz);
  fputs("OK\n",stdout);
}

void test_Xroots(uma sz,uma n) {
  uma i,j;
  uma ngx,ngy,nfx,nfy,nhx,nhy;
  R *buf1,*buf2,*f,*g,*h,*Q,*tmp,*xchg;
  struct ll_vec_lili *r1,*r2,*node;
  fputs("  root finding...",stdout);
  fflush(stdout);
  tmp = ll_vec_malloc_init(2 * sz);
  buf1 = ll_vec_malloc_init(sz * sz * sz);
  buf2 = ll_vec_malloc_init(sz * sz * sz);
  h = ll_vec_malloc_init(sz * sz * sz);
  Q = ll_vec_malloc_init(sz * sz * sz);
  for( j = 2 ; j < sz ; j++ ) {
    (void)n;
    for( i = 0 ; i < 5 ; i++ ) {
      uma k,nbroots;
      nbroots = 1 + (ll_random() % (j - 1));
      f = buf1;
      g = buf2;
      ll_vec_zero(f,j * j * j);
      R_castint(f[0],1);
      nfx = 1;
      nfy = 1;
      r1 = NULL;
      r2 = NULL;
      for( k = 1 ; k <= nbroots ; k++ ) {
        ll_vec_zero(tmp,2 * j);
        R_castint(tmp[j],-1);
        ll_upol_random(tmp,j);
        node = ll_vec_lili_node_malloc_copy(tmp,j);
        if ( r1 ) ll_vec_lili_node_attach_before(r1,node);
        r1 = node;
        ll_dbpol_mul(g,f,nfx,nfy,tmp,j,2);
        nfx += j - 1;
        nfy += 2 - 1;
        xchg = g;
        g = f;
        f = xchg;
      }
      ngx = j * j - nfx + 1;
      ngy = j - nfy + 1;
      ll_upol_random(g,ngx * ngy);
      ll_dbpol_mul(h,f,nfx,nfy,g,ngx,ngy);
      nhx = ngx + nfx - 1;
      nhy = ngy + nfy - 1;
      ll_mat_transpose(Q,h,nhy,nhx);
      r2 = ll_dbpol_Xroots(Q,nhy,nhx,j);
      for( node = r1 ; node ; node = node->next ) {
        if ( !ll_vec_lili_list_in(r2,node->a,node->na) ) {
          ll_print(stdout,"FAIL:\n\na must be a root of h\n"
                          "h = %q\n\na = %p\n",
                          h,j * j,j,node->a,node->na);
          exit(-1);
        }
      }
      ll_vec_lili_list_free(r1);
      ll_vec_lili_list_free(r2);
    }
  }
  ll_vec_free(tmp,2 * sz);
  ll_vec_free(buf1,sz * sz * sz);
  ll_vec_free(buf2,sz * sz * sz);
  ll_vec_free(h,sz * sz * sz);
  ll_vec_free(Q,sz * sz * sz);
  fputs("OK\n",stdout);
}

