/*
 * This file is part of decoding
 * Copyright 2012-2013  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#define _POSIX_C_SOURCE 1

#include <stdio.h>
#include <time.h>

#define __DECODING_KEEP_MACROS
#include <decoding/decoding.h>
#include "rings_for_arithmetic.h"

#ifndef N /* number of iterations */
#define N 50 
#endif
#ifndef SIZE /* size of polynomials */
#define SIZE 128
#endif
#ifndef SIZE_MUL /* size of polynomials for naive mul */
#define SIZE_MUL 64
#endif

/* macros */

#define test_print_r __C(R,_test_print_r)
#define test_sub __C(R,_test_sub)
#define test_neg __C(R,_test_neg)
#define test_add __C(R,_test_add)
#define test_mul_naive __C(R,_test_mul_naive)
#define test_mul __C(R,_test_mul)
#define test_shift_naive __C(R,_test_shift_naive)
#define test_shift __C(R,_test_shift)
#define test_eval_horner __C(R,_test_eval_horner)
#define test_gcd __C(R,_test_gcd)
#define test_roots __C(R,_test_roots)
#define test_eea __C(R,_test_eea)
#define test_partial_eea __C(R,_test_partial_eea)
#define test_interpolation_lagrange \
  __C(R,_test_interpolation_lagrange)

/* applications of macros */

#define R gfp_char
#include "ll_upol_tests.c"
#undef R
#define R gfp_short
#include "ll_upol_tests.c"
#undef R
#define R gfp_int
#include "ll_upol_tests.c"
#undef R
#define R gfp_long
#include "ll_upol_tests.c"
#undef R
#define R gf2n_char
#include "ll_upol_tests.c"
#undef R
#define R gf2n_short
#include "ll_upol_tests.c"
#undef R
#define R gf2n_int
#include "ll_upol_tests.c"
#undef R
#define R gf2n_long
#include "ll_upol_tests.c"
#undef R
/*
#define R gfp_dynamic_char
#include "ll_upol_tests.c"
#undef R
#define R gfp_dynamic_short
#include "ll_upol_tests.c"
#undef R
#define R gfp_dynamic_int
#include "ll_upol_tests.c"
#undef R
#define R gfp_dynamic_long
#include "ll_upol_tests.c"
#undef R
*/
#define R mpfq_gf2_16
#include "ll_upol_tests.c"
#undef R
#define R gfp_float
#include "ll_upol_tests.c"
#undef R
#define R gfp_double
#include "ll_upol_tests.c"
#undef R

/* list of tests */

#define TEST(_R,_SIZE,_SIZE_MUL,_N,_COMM) \
{ \
  __C(_R,_test_print_r)(); \
  __C(_R,_test_add)(_SIZE,_N); \
  __C(_R,_test_neg)(_SIZE,_N); \
  __C(_R,_test_sub)(_SIZE,_N); \
  __C(_R,_test_mul_naive)(_SIZE_MUL,_N,_COMM); \
  __C(_R,_test_mul)(_SIZE,_N); \
  __C(_R,_test_shift_naive)(_SIZE_MUL,_N,_COMM); \
  __C(_R,_test_shift)(_SIZE,_N); \
  __C(_R,_test_eval_horner)(_SIZE,_N,_COMM); \
  __C(_R,_test_gcd)(_SIZE_MUL,_N); \
  __C(_R,_test_roots)(_SIZE_MUL,_N); \
  __C(_R,_test_eea)(_SIZE_MUL,_N); \
  __C(_R,_test_partial_eea)(_SIZE_MUL,_N); \
  __C(_R,_test_interpolation_lagrange)(_SIZE_MUL,_N); \
}

int main(void) {
  ll_random_seed(time(NULL));

  /* static rings */

  gfp_char_ring_init();
  TEST(gfp_char,SIZE,SIZE_MUL,N,1)
  gfp_char_ring_clear();
  
  gfp_short_ring_init();
  TEST(gfp_short,SIZE,SIZE_MUL,N,1)
  gfp_short_ring_clear();

  gfp_int_ring_init();
  TEST(gfp_int,SIZE,SIZE_MUL,N,1)
  gfp_int_ring_clear();
  
  gfp_long_ring_init();
  TEST(gfp_long,SIZE,SIZE_MUL,N,1)
  gfp_long_ring_clear();
  
  gf2n_char_ring_init();
  TEST(gf2n_char,SIZE,SIZE_MUL,N,1)
  gf2n_char_ring_clear();
  
  gf2n_short_ring_init();
  TEST(gf2n_short,SIZE,SIZE_MUL,N,1)
  gf2n_short_ring_clear();
  
  gf2n_int_ring_init();
  TEST(gf2n_int,SIZE,SIZE_MUL,N,1)
  gf2n_int_ring_clear();
  
  gf2n_long_ring_init();
  TEST(gf2n_long,SIZE,SIZE_MUL,N,1)
  gf2n_long_ring_clear();
  
  gfp_float_ring_init();
  TEST(gfp_float,SIZE,SIZE_MUL,N,1)
  gfp_float_ring_clear();

  gfp_double_ring_init();
  TEST(gfp_double,SIZE,SIZE_MUL,N,1)
  gfp_double_ring_clear();
  
  /* dynamic rings */
  /*
  gfp_dynamic_char_ring_init(5);
  TEST(gfp_dynamic_char,SIZE,SIZE_MUL,N,1)
  gfp_dynamic_char_ring_clear();
  
  gfp_dynamic_short_ring_init(31);
  TEST(gfp_dynamic_short,SIZE,SIZE_MUL,N,1)
  gfp_dynamic_short_ring_clear();
  
  gfp_dynamic_int_ring_init(101);
  TEST(gfp_dynamic_int,SIZE,SIZE_MUL,N,1)
  gfp_dynamic_int_ring_clear();

  gfp_dynamic_long_ring_init(257);
  TEST(gfp_dynamic_long,SIZE,SIZE_MUL,N,1)
  gfp_dynamic_long_ring_clear();
  */
  
  /* external rings */
  
  mpfq_gf2_16_ring_init();
  TEST(mpfq_gf2_16,SIZE,SIZE_MUL,N,1)
  mpfq_gf2_16_ring_clear();
  
  return 0;
}

