/*
 * This file is part of decoding
 * Copyright 2012-2013  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

void test_print_r(void) {
  fputs("testing polynomials over " __S(R) "\n",stdout);
}

void test_sub(uma sz,uma n) {
  uma i,j;
  R *a,*b,*c,*ab;
  fputs("  substraction...",stdout);
  fflush(stdout);
  a = ll_vec_malloc_init(sz);
  b = ll_vec_malloc_init(sz);
  c = ll_vec_malloc_init(sz);
  ab = ll_vec_malloc_init(sz);
  for( j = 1 ; j < sz ; j++ ) {
    for( i = 0 ; i < n ; i++ ) {
      ll_vec_random(a,j);
      ll_vec_random(b,j);
      ll_vec_sub(ab,a,b,j);
      ll_vec_add(c,ab,b,j);
      if ( ll_vec_cmp(c,a,j) ) {
        ll_print(stdout,"FAIL:\n\n(a - b) + b != a\n"
                        "a = %p\n\nb = %p\n",a,j,b,j);
        exit(-1);
      }
    }
  }
  ll_vec_free(a,sz);
  ll_vec_free(b,sz);
  ll_vec_free(c,sz);
  ll_vec_free(ab,sz);
  fputs("OK\n",stdout);
}

void test_neg(uma sz,uma n) {
  uma i,j;
  R *a,*b,*c,*zero;
  fputs("  opposite...",stdout);
  fflush(stdout);
  a = ll_vec_malloc_init(sz);
  b = ll_vec_malloc_init(sz);
  c = ll_vec_malloc_init(sz);
  zero = ll_vec_malloc_init(sz);
  for( j = 1 ; j < sz ; j++ ) {
    for( i = 0 ; i < n ; i++ ) {
      ll_vec_random(a,j);
      ll_vec_neg(b,a,j);
      ll_vec_add(c,a,b,j);
      if ( ll_vec_cmp(c,zero,j) ) {
        ll_print(stdout,"FAIL:\n\na + (-a) != 0\n"
                        "a = %p\n",a,j);
        exit(-1);
      }
    }
  }
  ll_vec_free(a,sz);
  ll_vec_free(b,sz);
  ll_vec_free(c,sz);
  ll_vec_free(zero,sz);
  fputs("OK\n",stdout);
}

void test_add(uma sz,uma n) {
  uma i,j;
  R *a,*b,*c,*ab,*ba,*bc,*abc1,*abc2,*zero;
  fputs("  addition...",stdout);
  fflush(stdout);
  a = ll_vec_malloc_init(sz);
  b = ll_vec_malloc_init(sz);
  c = ll_vec_malloc_init(sz);
  ab = ll_vec_malloc_init(sz);
  ba = ll_vec_malloc_init(sz);
  bc = ll_vec_malloc_init(sz);
  abc1 = ll_vec_malloc_init(sz);
  abc2 = ll_vec_malloc_init(sz);
  zero = ll_vec_malloc_init(sz);
  for( j = 1 ; j < sz ; j++ ) {
    for( i = 0 ; i < n ; i++ ) {
      ll_vec_random(a,j);
      ll_vec_random(b,j);
      ll_vec_random(c,j);
      ll_vec_add(ab,a,b,j);
      ll_vec_add(ba,b,a,j);
      if ( ll_vec_cmp(ab,ba,j) ) {
        ll_print(stdout,"FAIL:\n\na * b != b * a\n"
                        "a = %p\n\nb = %p\n",a,j,b,j);
        exit(-1);
      }
      ll_vec_add(bc,b,c,j);
      ll_vec_add(abc1,ab,c,j);
      ll_vec_add(abc2,a,bc,j);
      if ( ll_vec_cmp(abc1,abc2,j) ) {
        ll_print(stdout,"FAIL:\n\n(a + b) + c != a + (b + c)\n"
                        "a = %p\n\nb = %p\n\nc = %p\n",a,j,b,j,c,j);
        exit(-1);
      }
      ll_vec_add(b,zero,a,j);
      if ( ll_vec_cmp(a,b,j) ) {
        ll_print(stdout,"FAIL:\n\na + 0 != a\n"
                        "a = %p\n",a,j);
        exit(-1);
      }
    }
  }
  ll_vec_free(a,sz);
  ll_vec_free(b,sz);
  ll_vec_free(c,sz);
  ll_vec_free(ab,sz);
  ll_vec_free(ba,sz);
  ll_vec_free(bc,sz);
  ll_vec_free(abc1,sz);
  ll_vec_free(abc2,sz);
  ll_vec_free(zero,sz);
  fputs("OK\n",stdout);
}

void test_mul_naive(uma sz,uma n,uma comm) {
  uma i,j;
  R *a,*b,*c,*ab,*ba,*bc,*ac,*abc1,*abc2,*one;
  fputs("  naive multiplication...",stdout);
  fflush(stdout);
  a = ll_vec_malloc_init(sz);
  b = ll_vec_malloc_init(sz);
  c = ll_vec_malloc_init(sz);
  ab = ll_vec_malloc_init(2 * sz);
  ba = ll_vec_malloc_init(2 * sz);
  bc = ll_vec_malloc_init(2 * sz);
  ac = ll_vec_malloc_init(2 * sz);
  abc1 = ll_vec_malloc_init(3 * sz);
  abc2 = ll_vec_malloc_init(3 * sz);
  one = ll_vec_malloc_init(sz);
  R_castint(one[0],1);
  for( j = 1 ; j < sz ; j++ ) {
    for( i = 0 ; i < n ; i++ ) {
      ll_vec_random(a,j);
      ll_vec_random(b,j);
      ll_vec_random(c,j);
      if ( comm ) {
        ll_upol_mul_naive(ab,a,j,b,j);
        ll_upol_mul_naive(ba,b,j,a,j);
        if ( ll_vec_cmp(ab,ba,2 * j - 1) ) {
          ll_print(stdout,"FAIL:\n\na * b != b * a\n"
                          "a = %p\n\nb = %p\n",a,j,b,j);
          exit(-1);
        }
      }
      ll_upol_mul_naive(ab,a,j,b,j);
      ll_upol_mul_naive(bc,b,j,c,j);
      ll_upol_mul_naive(abc1,ab,2 * j - 1,c,j);
      ll_upol_mul_naive(abc2,a,j,bc,2 * j - 1);
      if ( ll_vec_cmp(abc1,abc2,3 * j - 2) ) {
        ll_print(stdout,"FAIL:\n\n(a * b) * c != a * (b * c)\n"
                        "a = %p\n\nb = %p\n\nc = %p\n",a,j,b,j,c,j);
        exit(-1);
      }
      ll_vec_add(ab,a,b,j);
      ll_upol_mul_naive(abc1,ab,j,c,j);
      ll_upol_mul_naive(ac,a,j,c,j);
      ll_upol_mul_naive(bc,b,j,c,j);
      ll_vec_add(abc2,ac,bc,2 * j - 1);
      if ( ll_vec_cmp(abc1,abc2,2 * j - 1) ) {
        ll_print(stdout,"FAIL:\n\n(a + b) * c != (a * c) + (b * c)\n"
                        "a = %p\n\nb = %p\n\nc = %p\n",
                        a,j,b,j,c,j);
        exit(-1);
      }
      ll_upol_mul_naive(ab,one,j,a,j);
      if ( ll_vec_cmp(ab,a,j) ||
           !ll_vec_iszero(&ab[j],j - 1) ) {
        ll_print(stdout,"FAIL:\n\na * 1 != a\na = %p",a,j);
        exit(-1);
      }
    }
  }
  ll_vec_free(a,sz);
  ll_vec_free(b,sz);
  ll_vec_free(c,sz);
  ll_vec_free(ab,2 * sz);
  ll_vec_free(ba,2 * sz);
  ll_vec_free(bc,2 * sz);
  ll_vec_free(ac,2 * sz);
  ll_vec_free(abc1,3 * sz);
  ll_vec_free(abc2,3 * sz);
  ll_vec_free(one,sz);
  fputs("OK\n",stdout);
}

void test_mul(uma sz,uma n) {
  uma i,j;
  R *a,*b,*ab1,*ab2;
  fputs("  multiplication...",stdout);
  fflush(stdout);
  a = ll_vec_malloc_init(sz);
  b = ll_vec_malloc_init(sz);
  ab1 = ll_vec_malloc_init(2 * sz);
  ab2 = ll_vec_malloc_init(2 * sz);
  for( j = 1 ; j < sz ; j++ ) {
    for( i = 0 ; i < n ; i++ ) {
      ll_vec_random(a,j);
      ll_vec_random(b,j);
      ll_upol_mul_naive(ab1,a,j,b,j);
      ll_upol_mul(ab2,a,j,b,j);
      if ( ll_vec_cmp(ab1,ab2,2 * j - 1) ) {
        ll_print(stdout,"FAIL:\n\na * b != a * b\n\n"
                        "a = %p\n\nb = %p\n",a,j,b,j);
        exit(-1);
      }
    }
  }
  ll_vec_free(a,sz);
  ll_vec_free(b,sz);
  ll_vec_free(ab1,2 * sz);
  ll_vec_free(ab2,2 * sz);
  fputs("OK\n",stdout);
}

void test_shift_naive(uma sz,uma n,uma comm) {
  uma i,j;
  R a,b,*f1,*f2,*f3,*g1,*g2,*g3,*g4;
  fputs("  naive shift...",stdout);
  fflush(stdout);
  R_inits(a,b,NULL);
  f1 = ll_vec_malloc_init(sz);
  f2 = ll_vec_malloc_init(sz);
  f3 = ll_vec_malloc_init(2 * sz);
  g1 = ll_vec_malloc_init(2 * sz);
  g2 = ll_vec_malloc_init(2 * sz);
  g3 = ll_vec_malloc_init(2 * sz);
  g4 = ll_vec_malloc_init(2 * sz);
  for( j = 1 ; j < sz ; j++ ) {
    for( i = 0 ; i < n ; i++ ) {
      R_random(a);
      R_random(b);
      ll_vec_random(f1,j);
      ll_upol_shift_naive(g1,f1,j,a);
      ll_vec_mul_by_cte(g1,g1,j,b);
      ll_vec_mul_by_cte(f2,f1,j,b);
      ll_upol_shift_naive(g2,f2,j,a);
      if ( ll_vec_cmp(g1,g2,j) ) {
        ll_print(stdout,"FAIL:\n\n(a f1)(X + b) != a f1(X + b)\n"
                        "a = %e\nb = %e\nf1 = %p\n",a,b,f1,j);
        exit(-1);
      }
      ll_vec_random(f1,j);
      ll_vec_random(f2,j);
      ll_vec_add(f3,f1,f2,j);
      ll_upol_shift_naive(g3,f3,j,a);
      ll_upol_shift_naive(g1,f1,j,a);
      ll_upol_shift_naive(g2,f2,j,a);
      ll_vec_add(g1,g1,g2,j);
      if ( ll_vec_cmp(g1,g3,j) ) {
        ll_print(stdout,"FAIL:\n\n"
                        "(f1 + f2)(X + a) != f1(X + a) + f2(X + a)\n"
                        "a = %e\nf1 = %p\n\nf2 = %p\n",a,f1,j,f2,j);
        exit(-1);
      }
      if ( comm ) {
        ll_vec_random(f1,j);
        ll_vec_random(f2,j);
        ll_upol_mul(f3,f1,j,f2,j);
        ll_upol_shift_naive(g3,f3,2 * j - 1,a);
        ll_upol_shift_naive(g1,f1,j,a);
        ll_upol_shift_naive(g2,f2,j,a);
        ll_upol_mul(g4,g1,j,g2,j);
        if ( ll_vec_cmp(g3,g4,2 * j - 1) ) {
          ll_print(stdout,"FAIL:\n\n"
                          "(f1 * f2)(X + a) != f1(X + a) * f2(X + a)\n"
                          "a = %e\nf1 = %p\n\nf2 = %p\n",a,f1,j,f2,j);
          exit(-1);
        }        
      }
    }
  }
  R_clears(a,b,NULL);
  ll_vec_free(f1,sz);
  ll_vec_free(f2,sz);
  ll_vec_free(f3,2 * sz);
  ll_vec_free(g1,2 * sz);
  ll_vec_free(g2,2 * sz);
  ll_vec_free(g3,2 * sz);
  ll_vec_free(g4,2 * sz);
  fputs("OK\n",stdout);
}

void test_shift(uma sz,uma n) {
  uma i,j;
  R a,*f1,*g1,*g2;
  fputs("  shift...",stdout);
  fflush(stdout);
  R_init(a);
  f1 = ll_vec_malloc_init(sz);
  g1 = ll_vec_malloc_init(sz);
  g2 = ll_vec_malloc_init(sz);
  for( j = 1 ; j < sz ; j++ ) {
    for( i = 0 ; i < n ; i++ ) {
      R_random(a);
      ll_vec_random(f1,j);
      ll_upol_shift_naive(g1,f1,j,a);
      ll_upol_shift(g2,f1,j,a);
      if ( ll_vec_cmp(g1,g2,j) ) {
        ll_print(stdout,"FAIL:\n\nf1(X + a) != f1(X + a)\n"
                        "a = %e\n\nf1 = %p\n",a,f1,j);
        exit(-1);
      }
    }
  }
  R_clear(a);
  ll_vec_free(f1,sz);
  ll_vec_free(g1,sz);
  ll_vec_free(g2,sz);
  fputs("OK\n",stdout);
}

void test_eval_horner(uma sz,uma n,uma comm) {
  uma i,j;
  R *f1,*f2,*f3,a,b,c1,c2;
  fputs("  Horner evaluation...",stdout);
  fflush(stdout);
  R_inits(a,b,c1,c2,NULL);
  f1 = ll_vec_malloc_init(sz);
  f2 = ll_vec_malloc_init(sz);
  f3 = ll_vec_malloc_init(2 * sz);
  for( j = 1 ; j < sz ; j++ ) {
    for( i = 0 ; i < n ; i++ ) {
      R_random(a);
      R_random(b);
      ll_vec_random(f1,j);
      ll_upol_eval_horner(c1,f1,j,a);
      R_mul(c1,b,c1);
      ll_vec_mul_by_cte(f2,f1,j,b);
      ll_upol_eval_horner(c2,f2,j,a);
      if ( R_cmp(c1,c2) ) {
        ll_print(stdout,"FAIL:\n\n(b f1)(a) != b f1(a)\n"
                        "a = %e\nb = %e\nf1 = %p\n",a,b,f1,j);
        exit(-1);
      }
      ll_vec_random(f2,j);
      ll_upol_eval_horner(c1,f1,j,a);
      ll_upol_eval_horner(c2,f2,j,a);
      R_add(c1,c1,c2);
      ll_vec_add(f3,f1,f2,j);
      ll_upol_eval_horner(c2,f3,j,a);
      if ( R_cmp(c1,c2) ) {
        ll_print(stdout,"FAIL:\n\n(f1 + f2)(a) != f1(a) + f2(a)\n"
                        "a = %e\nf1 = %p\nf2 = %p\n",a,f1,j,f2,j);
        exit(-1);
      }
      if ( comm ) {
        ll_upol_eval_horner(c1,f1,j,a);
        ll_upol_eval_horner(c2,f2,j,a);
        R_mul(c1,c1,c2);
        ll_upol_mul(f3,f1,j,f2,j);
        ll_upol_eval_horner(c2,f3,2 * j - 1,a);
        if ( R_cmp(c1,c2) ) {
          ll_print(stdout,"FAIL:\n\n(f1 * f2)(a) != f1(a) * f2(a)\n"
                          "a = %e\nf1 = %p\nf2 = %p\n",a,f1,j,f2,j);
          exit(-1);
        }
      }
    }
  }
  R_clears(a,b,c1,c2,NULL);
  ll_vec_free(f1,sz);
  ll_vec_free(f2,sz);
  ll_vec_free(f3,2 * sz);
  fputs("OK\n",stdout);
}

void test_roots(uma sz,uma n) {
  uma i,j,k,l,nr2;
  R *f,*g,*r1,*r2;
  fputs("  root-finding...",stdout);
  fflush(stdout);
  f = ll_vec_malloc_init(sz);
  g = ll_vec_malloc_init(sz);
  r1 = ll_vec_malloc_init(sz);
  r2 = ll_vec_malloc_init(sz);
  for( j = 2 ; j < sz ; j++ ) {
    for( i = 0 ; i < n ; i++) {
      uma nroots;
      nroots = j / 2;
      ll_vec_random(r1,nroots);
      ll_upol_prod_affine_neg(f,r1,nroots);
      nr2 = j - nroots;
      ll_vec_random(r2,nr2);
      if ( ll_vec_iszero(r2,nr2) ) R_castint(r2[0],1);
      ll_upol_mul(g,f,nroots + 1,r2,nr2);
      ll_upol_roots(r2,&nr2,g,j);
      for( k = 0 ; k < nroots ; k++ ) {
        for( l = 0 ; l < nr2 ; l++ ) {
          if ( !R_cmp(r1[k],r2[l]) ) goto test_roots_found;
        }
        ll_print(stdout,"FAIL:\n\nr1 = %v\n\nr2 = %v\n",r1,nroots,r2,nr2);
        exit(-1);
        test_roots_found:;
      }
    }
  }
  ll_vec_free(f,sz);
  ll_vec_free(g,sz);
  ll_vec_free(r1,sz);
  ll_vec_free(r2,sz);
  fputs("OK\n",stdout);
}

void test_gcd(uma sz,uma n) {
  uma i,j,nrem,ngcd;
  R *a,*b,*c,*ca,*cb,*gcd,*cgcd,*rem;
  fputs("  gcd...",stdout);
  fflush(stdout);
  a = ll_vec_malloc_init(sz);
  b = ll_vec_malloc_init(sz);
  c = ll_vec_malloc_init(sz);
  ca = ll_vec_malloc_init(2 * sz);
  cb = ll_vec_malloc_init(2 * sz);
  gcd = ll_vec_malloc_init(2 * sz);
  cgcd = ll_vec_malloc_init(2 * sz);
  rem = ll_vec_malloc_init(sz);
  for( j = 1 ; j < sz ; j++ ) {
    for( i = 0 ; i < n ; i++ ) {
      uma ncgcd;
      ll_upol_random(a,j);
      ll_upol_gcd(gcd,&ngcd,a,j,a,j);
      if ( ngcd != j || ll_vec_cmp(gcd,a,j) ) {
        ll_print(stdout,"FAIL:\n\ngcd(a,a) != a\n"
                        "a = %p\n",a,j);
      }      
      ll_upol_random(b,j);
      ll_upol_gcd(gcd,&ngcd,a,j,b,j);
      ll_upol_rem(rem,&nrem,a,j,gcd,ngcd);
      if ( !ll_upol_iszero(rem,nrem) ) {
        ll_print(stdout,"FAIL:\n\na mod gcd != 0\n"
                        "a = %p\n\ngcd = %p\n",a,j,gcd,ngcd);
        exit(-1);
      }
      ll_upol_rem(rem,&nrem,b,j,gcd,ngcd);
      if ( !ll_upol_iszero(rem,nrem) ) {
        ll_print(stdout,"FAIL:\n\nb mod gcd != 0\n"
                        "b = %p\n\ngcd = %p\n",b,j,gcd,ngcd);
        exit(-1);
      }
      ll_upol_random(c,j);
      ll_upol_mul(ca,c,j,a,j);
      ll_upol_mul(cb,c,j,b,j);
      ll_upol_mul(cgcd,c,j,gcd,ngcd);
      ncgcd = j + ngcd - 1;
      ll_upol_gcd(gcd,&ngcd,ca,2 * j - 1,cb,2 * j - 1);
      if ( ngcd != ncgcd || ll_vec_cmp(cgcd,gcd,ngcd) ) {
        ll_print(stdout,"FAIL:\n\nc * gcd(a,b) != gcd(c * a,c * b)\n\n"
                        "a = %p\n\nb = %p\n\nc = %p\n",a,j,b,j,c,j);
        exit(-1);
      }
    }
  }
  ll_vec_free(a,sz);
  ll_vec_free(b,sz);
  ll_vec_free(c,sz);
  ll_vec_free(ca,2 * sz);
  ll_vec_free(cb,2 * sz);
  ll_vec_free(gcd,2 * sz);
  ll_vec_free(cgcd,2 * sz);
  ll_vec_free(rem,sz);
  fputs("OK\n",stdout);
}

void test_eea(uma sz,uma n) {
  uma i,j,ngcd,neea,nu,nv,nbezout;
  R *a,*b,*u,*v,*ua,*vb,*gcd,*eea,*bezout;
  fputs("  eea...",stdout);
  fflush(stdout);
  a = ll_vec_malloc_init(sz);
  b = ll_vec_malloc_init(sz);
  u = ll_vec_malloc_init(sz);
  v = ll_vec_malloc_init(sz);
  ua = ll_vec_malloc_init(2 * sz);
  vb = ll_vec_malloc_init(2 * sz);
  bezout = ll_vec_malloc_init(2 * sz);
  gcd = ll_vec_malloc_init(sz);
  eea = ll_vec_malloc_init(sz);
  for( j = 1 ; j < sz ; j++ ) {
    for( i = 0 ; i < n ; i++ ) {
      uma nua,nvb;
      ll_upol_random(a,j);
      ll_upol_random(b,j);
      ll_upol_gcd(gcd,&ngcd,a,j,b,j);
      ll_upol_eea(eea,&neea,u,&nu,v,&nv,a,j,b,j);
      if ( neea != ngcd || ll_vec_cmp(gcd,eea,ngcd) ) {
        ll_print(stdout,"FAIL:\n\ngcd != eea\n\n"
                        "a = %p\n\nb = %p\n",a,j,b,j);
        exit(-1);
      }
      ll_upol_mul(ua,u,nu,a,j);
      nua = nu + j - 1;
      ll_upol_mul(vb,v,nv,b,j);
      nvb = nv + j - 1;
      ll_array_add(bezout,ua,nua,vb,nvb);
      nbezout = ll_vec_true_size(bezout,MAX(nua,nvb));
      ll_upol_normalize(bezout,nbezout);
      ll_upol_normalize(gcd,ngcd);
      if ( nbezout != ngcd || ll_vec_cmp(bezout,gcd,ngcd) ) {
        ll_print(stdout,"FAIL:\n\nu * a + v * b != gcd\n\n"
                        "a = %p\n\nb = %p\n",a,j,b,j);
        
        ll_print(stdout,"u = %p\n\nv = %p\n\n",u,nu,v,nv);
        ll_print(stdout,"bezout = %p\n\ngcd = %p\n\n",
                        bezout,nbezout,gcd,ngcd);
        
        exit(-1);
      }
    }
  }
  ll_vec_free(a,sz);
  ll_vec_free(b,sz);
  ll_vec_free(u,sz);
  ll_vec_free(v,sz);
  ll_vec_free(ua,2 * sz);
  ll_vec_free(vb,2 * sz);
  ll_vec_free(bezout,2 * sz);
  ll_vec_free(gcd,sz);
  ll_vec_free(eea,sz);
  fputs("OK\n",stdout);
}

void test_partial_eea(uma sz,uma n) {
  uma i,j,k,neea,nu,nv,nbezout;
  R *a,*b,*u,*v,*ua,*vb,*eea,*bezout;
  fputs("  partial eea...",stdout);
  fflush(stdout);
  a = ll_vec_malloc_init(sz);
  b = ll_vec_malloc_init(sz);
  u = ll_vec_malloc_init(sz);
  v = ll_vec_malloc_init(sz);
  ua = ll_vec_malloc_init(2 * sz);
  vb = ll_vec_malloc_init(2 * sz);
  bezout = ll_vec_malloc_init(2 * sz);
  eea = ll_vec_malloc_init(sz);
  for( j = 1 ; j < sz ; j++ ) {
    (void)n;
    for( i = 0 ; i < 5 ; i++ ) {
      for( k = -1 ; k < j ; k++ ) {
        uma nua,nvb;
        ll_upol_random(a,j);
        ll_upol_random(b,j);
        ll_upol_partial_eea(eea,&neea,u,&nu,v,&nv,a,j,b,j,k);
        ll_upol_mul(ua,u,nu,a,j);
        nua = nu + j - 1;
        ll_upol_mul(vb,v,nv,b,j);
        nvb = nv + j - 1;
        ll_array_add(bezout,ua,nua,vb,nvb);
        nbezout = ll_vec_true_size(bezout,MAX(nua,nvb));
        if ( nbezout != neea || ll_vec_cmp(bezout,eea,neea) ) {
          ll_print(stdout,"FAIL:\n\nu * a + v * b != eea\n\n"
                          "a = %p\n\nb = %p\n",a,j,b,j);
          exit(-1);
        }
      }
    }
  }
  ll_vec_free(a,sz);
  ll_vec_free(b,sz);
  ll_vec_free(u,sz);
  ll_vec_free(v,sz);
  ll_vec_free(ua,2 * sz);
  ll_vec_free(vb,2 * sz);
  ll_vec_free(bezout,2 * sz);
  ll_vec_free(eea,sz);
  fputs("OK\n",stdout);
}

void test_interpolation_lagrange(uma sz,uma n) {
  uma i,j,k,nf;
  R *x,*y,*f,*ev;
  fputs("  Lagrange interpolation...",stdout);
  fflush(stdout);
  x = ll_vec_malloc_init(sz);
  y = ll_vec_malloc_init(sz);
  f = ll_vec_malloc_init(sz);
  ev = ll_vec_malloc_init(sz);
  for( j = 1 ; j < sz ; j++ ) {
    for( i = 0 ; i < n ; i++ ) {
      uma nx;
      if ( mpz_cmp_si(R_card,0) == 0 ) {
        nx = j;
      }
      else {
        if ( mpz_cmp_si(R_card,j) >= 0 ) {
          nx = j;
        }
        else {
          nx = mpz_get_si(R_card);
        }
      }
      R_random(x[0]);
      R_random(y[0]);
      for( k = 1 ; k < nx ; k++ ) {
        R_next(x[k],x[k - 1]);
        R_random(y[k]);
      }
      ll_upol_interpolation(f,&nf,x,y,nx);
      ll_upol_multi_eval(ev,f,nf,x,nx);
      if ( ll_vec_cmp(ev,y,nx) ) {
        ll_print(stdout,"FAIL:\n\nx = %v\n\ny = %v\n",x,nx,y,nx);
        ll_print(stdout,"ev = %v\n\n",ev,nx);
        ll_print(stdout,"f = %p\n\n",f,nf);
        exit(-1);
      }
    }
  }
  ll_vec_free(x,sz);
  ll_vec_free(y,sz);
  ll_vec_free(f,sz);
  ll_vec_free(ev,sz);
  fputs("OK\n",stdout);
}

