/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#define _POSIX_C_SOURCE 1

#include <stdio.h>
#include <time.h>

#define __DECODING_KEEP_MACROS
#include <decoding/decoding.h>
#include "rings_for_arithmetic.h"

#ifndef N
#define N 1000 /* number of iterations */
#endif

/* macros */

#define test_print_r  __C(R,_test_print_r)
#define test_div      __C(R,_test_div)
#define test_sub      __C(R,_test_sub)
#define test_neg      __C(R,_test_neg)
#define test_inv      __C(R,_test_inv)
#define test_add      __C(R,_test_add)
#define test_mul      __C(R,_test_mul)

/* applications of macros */

#define R gfp_char
#include "ring_arith_tests.c"
#undef R
#define R gfp_short
#include "ring_arith_tests.c"
#undef R
#define R gfp_int
#include "ring_arith_tests.c"
#undef R
#define R gfp_long
#include "ring_arith_tests.c"
#undef R
#define R gf2n_char
#include "ring_arith_tests.c"
#undef R
#define R gf2n_short
#include "ring_arith_tests.c"
#undef R
#define R gf2n_int
#include "ring_arith_tests.c"
#undef R
#define R gf2n_long
#include "ring_arith_tests.c"
#undef R
/*
#define R gfp_dynamic_char
#include "ring_arith_tests.c"
#undef R
#define R gfp_dynamic_short
#include "ring_arith_tests.c"
#undef R
#define R gfp_dynamic_int
#include "ring_arith_tests.c"
#undef R
#define R gfp_dynamic_long
#include "ring_arith_tests.c"
#undef R
*/
#define R mpfq_gf2_16
#include "ring_arith_tests.c"
#undef R
#define R gfp_float
#include "ring_arith_tests.c"
#undef R
#define R gfp_double
#include "ring_arith_tests.c"
#undef R
#define R gfp_gmp
#include "ring_arith_tests.c"
#undef R
#define R rationals_gmp
#include "ring_arith_tests.c"
#undef R

/* list of tests */

#define TEST(_R,_N,_COMM) \
{ \
  __C(_R,_test_print_r)(); \
  __C(_R,_test_add)(_N); \
  __C(_R,_test_neg)(_N); \
  __C(_R,_test_sub)(_N); \
  __C(_R,_test_mul)(_N,_COMM); \
  __C(_R,_test_inv)(_N); \
  __C(_R,_test_div)(_N); \
}

int main(void) {
  ll_random_seed(time(0));

  /* static rings */
  gfp_char_ring_init();
  TEST(gfp_char,N,1)
  gfp_char_ring_clear();
  
  gfp_short_ring_init();
  TEST(gfp_short,N,1)
  gfp_short_ring_clear();
  
  gfp_int_ring_init();
  TEST(gfp_int,N,1)
  gfp_int_ring_clear();
  
  gfp_long_ring_init();
  TEST(gfp_long,N,1)
  gfp_long_ring_clear();
  
  gf2n_char_ring_init();
  TEST(gf2n_char,N,1)
  gf2n_char_ring_clear();
  
  gf2n_short_ring_init();
  TEST(gf2n_short,N,1)
  gf2n_short_ring_clear();
  
  gf2n_int_ring_init();
  TEST(gf2n_int,N,1)
  gf2n_int_ring_clear();
  
  gf2n_long_ring_init();
  TEST(gf2n_long,N,1)
  gf2n_long_ring_clear();

  gfp_float_ring_init();
  TEST(gfp_float,N,1)
  gfp_float_ring_clear();

  gfp_double_ring_init();
  TEST(gfp_double,N,1)
  gfp_double_ring_clear();

  rationals_gmp_ring_init();
  TEST(rationals_gmp,N,1)
  rationals_gmp_ring_clear();

  /* dynamic rings */
  /*
  gfp_dynamic_char_ring_init(5);
  TEST(gfp_dynamic_char,N,1)
  gfp_dynamic_char_ring_clear();
  
  gfp_dynamic_short_ring_init(31);
  TEST(gfp_dynamic_short,N,1)
  gfp_dynamic_short_ring_clear();
  
  gfp_dynamic_int_ring_init(101);
  TEST(gfp_dynamic_int,N,1)
  gfp_dynamic_int_ring_clear();
  
  gfp_dynamic_long_ring_init(257);
  TEST(gfp_dynamic_long,N,1)
  gfp_dynamic_long_ring_clear();
  */
  
  {
    mpz_t p;
    mpz_init_set_str(p,"9234876293847562934785692349",10);
    gfp_gmp_ring_init(p);
    TEST(gfp_gmp,N,1)
    gfp_gmp_ring_clear();
    mpz_clear(p);
  }
 
  /* external rings */
  mpfq_gf2_16_ring_init();
  TEST(mpfq_gf2_16,N,1)
  mpfq_gf2_16_ring_clear();
  
  return 0;
}

