/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

void test_print_r(void) {
  fputs("testing arithmetic of " __S(R) "\n",stdout);
}

void test_div(uma n) {
  uma i;
  R a,b,c,ab;
  fputs("  division...",stdout);
  fflush(stdout);
  R_inits(a,b,c,ab,NULL);
  for( i = 0 ; i < n ; i++ ) {
    R_random(a);
    R_random(b);
    if ( R_div(ab,a,b) ) continue;
    R_mul(c,ab,b);
    if ( R_cmp(c,a) ) {
      ll_print(stdout,"FAIL\n\n"
                      "%e = (%e / %e) * %e != %e\n",c,a,b,b,a);
      exit(-1);
    }
  }
  R_clears(a,b,c,ab,NULL);
  fputs("OK\n",stdout);
}

void test_sub(uma n) {
  uma i;
  R a,b,c,ab;
  fputs("  substraction...",stdout);
  fflush(stdout);
  R_inits(a,b,c,ab,NULL);
  for( i = 0 ; i < n ; i++ ) {
    R_random(a);
    R_random(b);
    R_sub(ab,a,b);
    R_add(c,ab,b);
    if ( R_cmp(c,a) ) {
      ll_print(stdout,"FAIL\n\n"
                      "%e = (%e - %e) + %e != %e\n",c,a,b,b,a);
      exit(-1);
    }
  }
  R_clears(a,b,c,ab,NULL);
  fputs("OK\n",stdout);
}

void test_neg(uma n) {
  uma i;
  R a,b,c;
  fputs("  opposite...",stdout);
  fflush(stdout);
  R_inits(a,b,c,NULL);
  for( i = 0 ; i < n ; i++ ) {
    R_random(a);
    R_neg(b,a);
    R_add(c,a,b);
    if ( !R_iszero(c) ) {
      ll_print(stdout,"FAIL\n\n%e + (-%e) != 0\n",a,b);
      exit(-1);
    }
  }
  R_clears(a,b,c,NULL);
  fputs("OK\n",stdout);
}

void test_inv(uma n) {
  uma i;
  R a,b,c,one;
  fputs("  inverse...",stdout);
  fflush(stdout);
  R_inits(a,b,c,one,NULL);
  R_castint(one,1);
  for( i = 0 ; i < n ; i++ ) {
    R_random(a);
    if ( !R_inv(b,a) ) {
      R_mul(c,a,b);
      if ( R_cmp(c,one) ) {
        ll_print(stdout,"FAIL\n\n%e * (%e)^-1 != 1\n",a,b);
        exit(-1);
      }
    }
  }
  R_clears(a,b,c,one,NULL);
  fputs("OK\n",stdout);
}

void test_add(uma n) {
  uma i;
  R a,b,c,ab,ba,bc,abc1,abc2,zero;
  fputs("  addition...",stdout);
  fflush(stdout);
  R_inits(a,b,c,ab,ba,bc,abc1,abc2,zero,NULL);
  R_castint(zero,0);
  for( i = 0 ; i < n ; i++ ) {
    R_random(a);
    R_random(b);
    R_random(c);
    R_add(ab,a,b);
    R_add(ba,b,a);
    if ( R_cmp(ab,ba) ) {
      ll_print(stdout,"FAIL\n\n"
                      "%e = %e + %e != %e + %e = %e\n",
                      ab,a,b,b,a,ba);
      exit(-1);
    }
    R_add(bc,b,c);
    R_add(abc1,ab,c);
    R_add(abc2,a,bc);
    if ( R_cmp(abc1,abc2) ) {
      ll_print(stdout,"FAIL\n\n"
               "%e = (%e + %e) + %e != %e + (%e + %e) = %e\n",
               abc1,a,b,c,a,b,c,abc2);
      exit(-1);
    }
    R_add(b,zero,a);
    if ( R_cmp(a,b) ) {
      ll_print(stdout,"FAIL\n\n%e = %e + 0 != %e\n",b,a,a);
      exit(-1);
    }
  }
  R_clears(a,b,c,ab,ba,bc,abc1,abc2,zero,NULL);
  fputs("OK\n",stdout);
}

void test_mul(uma n,uma comm) {
  uma i;
  R a,b,c,ab,ba,bc,ac,abc1,abc2,one;
  fputs("  multiplication...",stdout);
  fflush(stdout);
  R_inits(a,b,c,ab,ba,bc,ac,abc1,abc2,one,NULL);
  R_castint(one,1);
  for( i = 0 ; i < n ; i++ ) {
    R_random(a);
    R_random(b);
    R_random(c);
    if ( comm ) {
      R_mul(ab,a,b);
      R_mul(ba,b,a);
      if ( R_cmp(ab,ba) ) {
        ll_print(stderr,"Error\n%e = %e * %e != %e * %e = %e\n",
                        ab,a,b,b,a,ba);
        exit(-1);
      }
    }
    R_mul(ab,a,b);
    R_mul(bc,b,c);
    R_mul(abc1,ab,c);
    R_mul(abc2,a,bc);
    if ( R_cmp(abc1,abc2) ) {
      ll_print(stdout,"FAIL\n\n"
               "%e = (%e * %e) * %e != %e * (%e * %e) = %e\n",
               abc1,a,b,c,a,b,c,abc2);
      exit(-1);
    }
    R_add(ab,a,b);
    R_mul(abc1,ab,c);
    R_mul(ac,a,c);
    R_mul(bc,b,c);
    R_add(abc2,ac,bc);
    if ( R_cmp(abc1,abc2) ) {
      ll_print(stdout,"FAIL\n\n"
               "%e = (%e + %e) * %e != (%e * %e) + (%e * %e) = %e\n",
               abc1,a,b,c,a,c,b,c,abc2);
      exit(-1);
    }
    R_mul(b,one,a);
    if ( R_cmp(a,b) ) {
      ll_print(stdout,"FAIL\n\n%e = %e * 1 != %e\n",b,a,a);
      exit(-1);
    }
  }
  R_clears(a,b,c,ab,ba,bc,ac,abc1,abc2,one,NULL);
  fputs("OK\n",stdout);
}

