/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#define _POSIX_C_SOURCE 1

#include <stdio.h>

#define __DECODING_KEEP_MACROS
#include <decoding/decoding.h>
#include "rings_for_decoding.h"

#ifndef ITER /* number of iterations */
#define ITER 50
#endif
#ifndef N /* length of the code */
#define N 64
#endif
#ifndef K /* dimension of the code */
#define K 10
#endif

/* macros */

#define test_guruswami_sudan __C(R,_test_guruswami_sudan)
#define test_guruswami_sudan_good __C(R,_test_guruswami_sudan_good)

/* applications of macros */

#define R gfp_int
#include "sugar_guruswami_sudan_tests.c"
#undef R
#define R gfp_long
#include "sugar_guruswami_sudan_tests.c"
#undef R
#define R gf2n_short
#include "sugar_guruswami_sudan_tests.c"
#undef R
#define R gf2n_int
#include "sugar_guruswami_sudan_tests.c"
#undef R
#define R gf2n_long
#include "sugar_guruswami_sudan_tests.c"
#undef R
/*
#define R gfp_dynamic_int
#include "sugar_guruswami_sudan_tests.c"
#undef R
#define R gfp_dynamic_long
#include "sugar_guruswami_sudan_tests.c"
#undef R
*/
#define R mpfq_gf2_8
#include "sugar_guruswami_sudan_tests.c"
#undef R
#define R gfp_float
#include "sugar_guruswami_sudan_tests.c"
#undef R
#define R gfp_double
#include "sugar_guruswami_sudan_tests.c"
#undef R

/* list of tests */

#define TEST(_R,_N,_K,_ITER) \
{ \
  __C(_R,_test_guruswami_sudan)(_N,_K,90,_ITER); \
}

int main(void) {
  ll_random_seed(time(NULL));

  /* static rings */
  gfp_int_ring_init();
  TEST(gfp_int,N,K,ITER)
  gfp_int_ring_clear();
  
  gfp_long_ring_init();
  TEST(gfp_long,N,K,ITER)
  gfp_long_ring_clear();
  
  gf2n_short_ring_init();
  TEST(gf2n_short,N,K,ITER)
  gf2n_short_ring_clear();
  
  gf2n_int_ring_init();
  TEST(gf2n_int,N,K,ITER)
  gf2n_int_ring_clear();
  
  gf2n_long_ring_init();
  TEST(gf2n_long,N,K,ITER)
  gf2n_long_ring_clear();
  
  gfp_float_ring_init();
  TEST(gfp_float,N,K,ITER)
  gfp_float_ring_clear();
  
  gfp_double_ring_init();
  TEST(gfp_double,N,K,ITER)
  gfp_double_ring_clear();
   
  /* dynamic rings */
  /*  
  gfp_dynamic_int_ring_init(101);
  TEST(gfp_dynamic_int,N,K,ITER)
  gfp_dynamic_int_ring_clear();

  gfp_dynamic_long_ring_init(257);
  TEST(gfp_dynamic_long,N,K,ITER)
  gfp_dynamic_long_ring_clear();
  */
  
  /* external rings */
  mpfq_gf2_8_ring_init();
  TEST(mpfq_gf2_8,N,K,ITER)
  mpfq_gf2_8_ring_clear();

  return 0;
}

