##
This file is part of decoding
Copyright (C) 2013  Guillaume Quintin

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
##

==NAME==

cpuid - print some processor capabilities

==SYNOPSIS==

`cpuid` [`-v` | `-f` | `-h` `prefix`]

==DESCRIPTION==

Print to the standard output the vendor name of the
processor and some of its capabilities.
For now only the `Intel` and `AMD`, `x86` and `x86_64`
processors are supported. Each capability is written
on the standard output in a new line.

For now only the following capabilities are detected.

> `SSE`
  SSE instruction set support.

> `SSE2`
  SSE 2 instruction set support.

> `SSE3`
  SSE 3 instruction set support.

> `SSSE3`
  SSSE 3 instruction set support.

> `SSE41`
  SSE 4.1 instruction set support.

> `SSE42`
  SSE 4.2 instruction set support.

> `AVX`
  AVX instruction set support.

> `AES`
  AES instruction set support.

> `PCLMULQDQ`
  PCLMULQDQ instruction support.

`SSE` stands for Streaming SIMD Extensions.
`SSSE` stands for Supplemental Streaming SIMD Extension.
`AVX` stands for Advanced Vector Extensions.
`AES` stands for Advanced Encryption Standard.
`PCLMULQDQ` is the Carry-Less Multiplication Instruction.

> `-v`
  Print `cpuid`'s version and exit.

> `-f`
  Print compilation flags to give `gcc(1)` and compatible
  compilers. Here is a list of flags that will be printed.
  Only the flag corresponding to the last version of `SSE`
  is given.

> `-h` `prefix`
  Print C macros correponsding to the capabilities of your
  processor as follows.

  ``
  #define prefixSSE              /* If SSE capability */
  #define prefixSSE2             /* If SSE 2 capability */
  #define prefixSSE3             /* If SSE 3 capability */
  #define prefixSSSE3            /* If SSSE 3 capability */
  #define prefixSSE41            /* If SSE 4.1 capability */
  #define prefixSSE42            /* If SSE 4.2 capability */
  #define prefixAVX              /* If AVX capability */
  #define prefixAES              /* If AES capability */
  #define prefixPCLMULQDQ        /* If PCLMULQDQ capability */
  #define prefixPHYSICAL_CORES 1 /* Number of physical cores */
  #define prefixLOGICAL_CORES 1  /* Number of logical cores */
  ``

==AUTHOR==

Written by Guillaume Quintin (quintin@lix.polytechnique.fr).

==SEE ALSO==

`proc`(5),
`gcc`(1).
