##
This file is part of decoding
Copyright (C) 2012  Guillaume Quintin, INRIA

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
##

==NAME==

guruswami-sudan - print Guruswami-Sudan list decoding parameters

==SYNOPSIS==

`guruswami-sudan` `n` `k` [ `--all` | `--list-size` | \
`--max-errors` | `--max-errors` `e` ] ...

==DESCRIPTION==

Print Guruswami-Sudan list decoding algorithm parameters. By default
when invoked with the length of the code `n` and its
dimension `k` followed by `--all`, `guruswami-sudan` prints all the
informations about the Guruswami-Sudan list decoding algorithm:
the unique decoding radius of the code, the maximum number
of errors that can be corrected and the list-size which
corresponds to the maximum number of codewords that can be
returned.

> `n`
  The length of the code.

> `k`
  The dimension of the code.

> `--all`
  Print all the information about the Guruswami-Sudan algorithm.

> `--max-errors`
  If the multiplicity is given with `--mult` print
  the maximum number of correctible errors given
  that multiplicity.
  
  Otherwise print the maximum number of correctible errors.
  
  It is useful when `guruswami-sudan` is invoked
  within a shell script.

> `--list-size`
  If the multiplicity is given with `--mult` print
  the maximum number of correctible errors given
  that multiplicity.
  
  If the number of expected errors is given with `--errors` print
  the list-size given that number of expected errors.

  Otherwise print the list-size corresponding to the
  maximum number of correctible errors.
  
  It is useful when `guruswami-sudan` is invoked
  within a shell script.

> `--max-mult`
  If the number of expected errors is given with `--errors` print
  the maximum multiplicity given that number of expected errors.

  Otherwise print the multiplicity corresponding to the
  maximum number of correctible errors.

> `mult` `m`
  Indicate that one wants to correct errors with a curve
  with multiplicity at least `m` at the interpolation
  points.

> `--errors` `e`
  Indicate that one wants to correct `e` errors.

==EXEMPLES==

> `guruswami-sudan 50 10 --all`
  Print all informations related to a [50,10] Reed-Solomon
  code to be decoded by the Guruswami-Sudan algorithm.

> `guruswami-sudan 50 10 --errors 21 --all`
  Print all informations related to a [50,10] Reed-Solomon
  code to be decoded with an expected number of 21 errors
  by the Guruswami-Sudan algorithm.

> `guruswami-sudan 50 10 --errors 21 --list-size`
  Print the list-size related to a [50,10] Reed-Solomon
  code to be decoded with an expected number of 21 errors
  by the Guruswami-Sudan algorithm.

> `guruswami-sudan 50 10 --mult 2 --max-errors`
  Print the number of correctible errors for a [50,10]
  Reed-Solomon code by the Guruswami-Sudan algorithm
  given multiplicity 2 for the curve at the interpolating
  points.

==AUTHOR==

Written by Guillaume Quintin (quintin@lix.polytechnique.fr).

